@@ -15,6 +15,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#include "config.h"
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -22,11 +25,14 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <math.h>
-
 #include <glib.h>
-#include <gdk_imlib.h>
 
-#include "config.h"
+#ifdef HAVE_GDK_PIXBUF
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#else
+#include <gdk_imlib.h>
+#endif
+
 #ifdef GNOME
 #include <gnome.h>
 #endif
@@ -53,9 +59,16 @@
 #include "undo.h"
 #include "pagesetup.h"
 #include "dia_dirs.h"
+#include "focus.h"
+#include "gdk/gdk.h"
+#include "gdk/gdkkeysyms.h"
+#include "lib/properties.h"
 
+#ifdef HAVE_GDK_PIXBUF
+GdkPixbuf *logo;
+#else
 GdkImlibImage *logo;
-
+#endif
 
 void file_quit_callback(GtkWidget *widget, gpointer data)
 {
@@ -240,6 +253,130 @@ edit_paste_callback(GtkWidget *widget, gpointer data)
   diagram_flush(ddisp->diagram);
 }
 
+/* Signal handler for getting the selection from whoever */
+void
+received_selection_handler(GtkWidget *widget, GtkSelectionData *selection,
+			   gpointer data)
+{
+  Focus *focus = active_focus();
+  gchar *text;
+  int i;
+  ObjectChange *change = NULL;
+  int modified, any_modified = FALSE;
+  Object *obj;
+  DDisplay *ddisp = ddisplay_active();
+
+  if ((focus == NULL) || (!focus->has_focus)) return;
+
+  obj = focus->obj;
+
+  /* **** IMPORTANT **** Check to see if retrieval succeeded  */
+  if (selection->length < 0)
+  {
+    g_print ("Selection retrieval failed\n");
+    return;
+  }
+  /* Make sure we got the data in the expected form */
+  if (selection->type != GDK_SELECTION_TYPE_STRING)
+  {
+    g_print ("Selection \"STRING\" was not returned as a string!\n");
+    return;
+  }
+
+  text = (gchar *)selection->data;
+  for (i = 0; i < selection->length; i++) {
+    if (text[i] == '\n') {
+      modified = (*focus->key_event)(focus, GDK_Return, "\n", 1, &change);
+    } else {
+      modified = (*focus->key_event)(focus, GDK_A, &text[i], 1, &change);
+    }
+    { /* Make sure object updates its data: */
+      Point p = obj->position;
+      (obj->ops->move)(obj,&p);  }
+
+    /* Perhaps this can be improved */
+    object_add_updates(obj, ddisp->diagram);
+    
+    if (modified && (change != NULL)) {
+      undo_object_change(ddisp->diagram, obj, change);
+      any_modified = TRUE;
+    }
+    
+    diagram_flush(ddisp->diagram);
+  }
+
+  if (any_modified) 
+    undo_set_transactionpoint(ddisp->diagram->undo);
+}
+
+static char *current_clipboard;
+
+void
+get_selection_handler(GtkWidget *widget, GtkSelectionData *selection,
+		      gpointer data) {
+  if (current_clipboard) {
+    gtk_selection_data_set(selection, GDK_TARGET_STRING,
+			   8, current_clipboard, strlen(current_clipboard));
+  }
+}
+
+void
+edit_copy_text_callback(GtkWidget *widget, gpointer data)
+{
+  Focus *focus = active_focus();
+  DDisplay *ddisp;
+  Object *obj;
+  Property textprop;
+
+  if ((focus == NULL) || (!focus->has_focus)) return;
+
+  ddisp = ddisplay_active();
+
+  obj = focus->obj;
+
+  if (obj->ops->get_props == NULL) 
+    return;
+
+  textprop.name = "text";
+  textprop.type = PROP_TYPE_INVALID;
+  PROP_VALUE_STRING(textprop) = NULL;
+
+  /* Get the first text property */
+  obj->ops->get_props(obj, &textprop, 1);
+  
+  if (current_clipboard) g_free(current_clipboard);
+
+  if (textprop.type != PROP_TYPE_STRING)
+    return;
+
+  current_clipboard = g_strdup(PROP_VALUE_STRING(textprop));
+
+  prop_free(&textprop);
+
+  gtk_selection_owner_set(GTK_WIDGET(ddisp->shell),
+			  GDK_SELECTION_PRIMARY,
+			  GDK_CURRENT_TIME);
+  gtk_selection_add_target(GTK_WIDGET(ddisp->shell),
+			   GDK_SELECTION_PRIMARY,
+			   GDK_TARGET_STRING, 0);
+}
+
+void
+edit_cut_text_callback(GtkWidget *widget, gpointer data)
+{
+}
+
+void
+edit_paste_text_callback(GtkWidget *widget, gpointer data)
+{
+  DDisplay *ddisp;
+
+  ddisp = ddisplay_active();
+
+  gtk_selection_convert(ddisp->shell, GDK_SELECTION_PRIMARY, GDK_TARGET_STRING,
+			GDK_CURRENT_TIME);
+}
+
 void
 edit_delete_callback(GtkWidget *widget, gpointer data)
 {
@@ -296,14 +433,30 @@ help_about_callback(GtkWidget *widget, gpointer data)
 {
   GtkWidget *dialog;
   GtkWidget *vbox;
+  GtkWidget *table;
   GtkWidget *bbox;
   GtkWidget *frame;
   GtkWidget *label;
   GtkWidget *button;
   char str[100];
-
+  gint i;
+  
   GtkWidget *gpixmap;
 
+  static const gchar *contributors[] = {
+    "Jerome Abela",
+    "Emmanuel Briot",
+    "Cyrille Chepelov",
+    "Lars R. Clausen",
+    "Fredrik Hallenberg",
+    "Francis J. Lacoste",
+    "Steffen Macke",
+    "Jacek Pliszka",
+    "Henk Jan Priester",
+    "Alejandro Aguilar Sierra",
+  };
+  const gint ncontributors = sizeof(contributors) / sizeof(contributors[0]);
+
   dialog = gtk_dialog_new ();
   gtk_window_set_wmclass (GTK_WINDOW (dialog), "about_dialog", "Dia");
   gtk_window_set_title (GTK_WINDOW (dialog), _("About Dia"));
@@ -320,11 +473,27 @@ help_about_callback(GtkWidget *widget, gpointer data)
   if (!logo) {
       gchar* datadir = dia_get_data_directory(""); 
       g_snprintf(str, sizeof(str), "%s/dia_logo.png", datadir);
+#ifdef HAVE_GDK_PIXBUF
+      logo = gdk_pixbuf_new_from_file(str);
+#else
       logo = gdk_imlib_load_image(str);
+#endif
       g_free(datadir);
   }
 
   if (logo) {
+#ifdef HAVE_GDK_PIXBUF
+    GdkPixmap *pixmap;
+    GdkBitmap *bitmap;
+
+      frame = gtk_frame_new (NULL);
+      gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
+      gtk_container_set_border_width (GTK_CONTAINER (frame), 1);
+      gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 1);
+
+      gdk_pixbuf_render_pixmap_and_mask(logo, &pixmap, &bitmap, 128);
+      gpixmap = gtk_pixmap_new(pixmap, bitmap);
+#else
       gdk_imlib_render(logo, logo->rgb_width, logo->rgb_height);
       frame = gtk_frame_new (NULL);
       gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
@@ -332,6 +501,7 @@ help_about_callback(GtkWidget *widget, gpointer data)
       gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 1);
 
       gpixmap = gtk_pixmap_new(logo->pixmap, logo->shape_mask);
+#endif
 
       gtk_container_add (GTK_CONTAINER(frame), gpixmap);
   }
@@ -341,17 +511,35 @@ help_about_callback(GtkWidget *widget, gpointer data)
   gtk_container_set_border_width (GTK_CONTAINER (frame), 1);
   gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 1);
 
-  vbox = gtk_vbox_new (FALSE, 1);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 1);
-  gtk_container_add (GTK_CONTAINER (frame), vbox);
+  table = gtk_table_new(3, 2, FALSE);
+  gtk_container_set_border_width (GTK_CONTAINER (table), 1);
+  gtk_container_add (GTK_CONTAINER (frame), table);
 
   g_snprintf(str, sizeof(str), _("Dia v %s by Alexander Larsson"), VERSION);
   label = gtk_label_new (str);
-  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 2);
+  gtk_table_attach(GTK_TABLE(table), label, 0,2, 0,1,
+		   GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
+  
+  label = gtk_label_new(_("Maintainer: James Henstridge"));
+  gtk_table_attach(GTK_TABLE(table), label, 0,2, 1,2,
+		   GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
 
   label = gtk_label_new (_("Please visit http://www.lysator.liu.se/~alla/dia "
 			   "for more info"));
-  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 2);
+  gtk_table_attach(GTK_TABLE(table), label, 0,2, 2,3,
+		   GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
+
+  label = gtk_label_new (_("Contributors:"));
+  gtk_table_attach(GTK_TABLE(table), label, 0,2, 3,4,
+		   GTK_FILL|GTK_EXPAND, GTK_FILL, 0,2);
+
+  for (i = 0; i < ncontributors; i++) {
+    label = gtk_label_new(contributors[i]);
+    gtk_table_attach(GTK_TABLE(table), label, i%2,i%2+1, i/2+4,i/2+5,
+		   GTK_FILL|GTK_EXPAND, GTK_FILL, 0,0);
+  }
+  gtk_table_set_col_spacings(GTK_TABLE(table), 1);
+  gtk_table_set_row_spacings(GTK_TABLE(table), 1);
 
   bbox = gtk_hbutton_box_new();
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), bbox, TRUE, TRUE, 5);
