@@ -30,18 +30,19 @@
 #include "properties.h"
 #include "cut_n_paste.h"
 #include "layer_dialog.h"
+#include "app_procs.h"
+#include "dia_dirs.h"
 
 GList *open_diagrams = NULL;
 
 Diagram *
-new_diagram(char *filename)  /* Note: filename is copied */
+new_diagram(const char *filename)  /* Note: filename is copied */
 {
   Diagram *dia = g_new(Diagram, 1);
-
+  
   dia->data = new_diagram_data();
 
-  dia->filename = g_malloc(strlen(filename)+1);
-  strcpy(dia->filename, filename);
+  dia->filename = g_strdup(filename);
   
   dia->unsaved = TRUE;
   dia->modified = FALSE;
@@ -59,12 +60,27 @@ new_diagram(char *filename)  /* Note: filename is copied */
 void
 diagram_destroy(Diagram *dia)
 {
+  char *home_path;
+
   assert(dia->displays==NULL);
 
   diagram_data_destroy(dia->data);
   
   g_free(dia->filename);
 
+  /* Save menu accelerators */
+  home_path = dia_config_filename("menus" G_DIR_SEPARATOR_S "display");
+
+  if (home_path != NULL) {
+    GtkPatternSpec pattern;
+
+    gtk_pattern_spec_init(&pattern, "*<Display>*");
+
+    gtk_item_factory_dump_rc (home_path, &pattern, TRUE);
+    g_free (home_path);
+    gtk_pattern_spec_free_segs(&pattern);
+  }
+
   open_diagrams = g_list_remove(open_diagrams, dia);
   layer_dialog_update_diagram_list();
 
@@ -107,6 +123,9 @@ void diagram_update_menu_sensitivity(Diagram *dia)
 #ifndef GNOME
   static GtkWidget *delete;
 #endif
+  static GtkWidget *copy_text;
+  static GtkWidget *cut_text;
+  static GtkWidget *paste_text;
 
   static GtkWidget *send_to_back;
   static GtkWidget *bring_to_front;
@@ -144,6 +163,13 @@ void diagram_update_menu_sensitivity(Diagram *dia)
     delete = menus_get_item_from_path(path->str);
 #   endif
 
+    g_string_append (g_string_assign(path, display),_("/Edit/Copy Text"));
+    copy_text = menus_get_item_from_path(path->str);
+    g_string_append (g_string_assign(path, display),_("/Edit/Cut Text"));
+    cut_text = menus_get_item_from_path(path->str);
+    g_string_append (g_string_assign(path, display),_("/Edit/Paste Text"));
+    paste_text = menus_get_item_from_path(path->str);
+
     g_string_append (g_string_assign(path, display),_("/Objects/Send to Back"));
     send_to_back = menus_get_item_from_path(path->str);
     g_string_append (g_string_assign(path, display),_("/Objects/Bring to Front"));
@@ -186,6 +212,10 @@ void diagram_update_menu_sensitivity(Diagram *dia)
   gtk_widget_set_sensitive(delete, dia->data->selected_count > 0);
 #endif
 
+  gtk_widget_set_sensitive(copy_text, active_focus() != NULL);
+  gtk_widget_set_sensitive(cut_text, 0); /* Not implemented */
+  gtk_widget_set_sensitive(paste_text, active_focus() != NULL);
+
   gtk_widget_set_sensitive(send_to_back, dia->data->selected_count > 0);
   gtk_widget_set_sensitive(bring_to_front, dia->data->selected_count > 0);
   
@@ -221,7 +251,10 @@ diagram_remove_ddisplay(Diagram *dia, DDisplay *ddisp)
   dia->display_count--;
 
   if(dia->display_count == 0) {
-    diagram_destroy(dia);
+    if (!app_is_embedded()) {
+      /* Don't delete embedded diagram when last view is closed */
+      diagram_destroy(dia);
+    }
   }
 }
 
@@ -668,8 +701,6 @@ diagram_place_under_selected(Diagram *dia)
   GList *sorted_list;
   GList *orig_list;
 
-  printf("diagram_place_under_selected()\n");
-  
   if (dia->data->selected_count == 0)
     return;
 
@@ -692,8 +723,6 @@ diagram_place_over_selected(Diagram *dia)
   GList *sorted_list;
   GList *orig_list;
 
-  printf("diagram_place_over_selected()\n");
-  
   if (dia->data->selected_count == 0)
     return;
 
