@@ -45,7 +45,7 @@ struct _Diagram {
 
 extern GList *open_diagrams; /* Read only! */
 
-extern Diagram *new_diagram(char *filename);  /* Note: filename is copied */
+extern Diagram *new_diagram(const char *filename); /*Note: filename is copied*/
 extern void diagram_destroy(Diagram *dia);
 extern void diagram_modified(Diagram *dia);
 extern void diagram_set_modified(Diagram *dia, int modified);
