@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -40,7 +40,7 @@ struct _Grid {
 
 #include "display.h"
 
-extern void grid_draw(DDisplay *ddisp);
+extern void grid_draw(DDisplay *ddisp, Rectangle *update);
 extern void snap_to_grid(Grid *grid, coord *x, coord *y);
 extern void grid_show_dialog(Grid *grid, DDisplay *ddisp);
 extern void grid_destroy_dialog(Grid *grid);
