@@ -15,103 +15,92 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-/* $Header: /cvs/gnome/dia/app/interface.c,v 1.26 2000/02/20 07:40:21 jamesh Exp $ */
+/* $Header: /cvs/gnome/dia/app/interface.c,v 1.35 2000/05/09 11:49:17 jamesh Exp $ */
 
 #include "config.h"
-#define USE_WRAPBOX
 
-#ifdef GNOME
-#include <gnome.h>
-#endif
-#ifdef USE_WRAPBOX
-#include "gtkhwrapbox.h"
-#endif
-#include "intl.h"
 #include "interface.h"
-#include "menus.h"
-#include "disp_callbacks.h"
-#include "tool.h"
-#include "sheet.h"
-#include "app_procs.h"
-#include "arrows.h"
-#include "color_area.h"
-#include "linewidth_area.h"
-#include "lineprops_area.h"
-#include "attributes.h"
 
 #include "pixmaps.h"
 
-typedef struct _ToolButton ToolButton;
+#include "preferences.h"
 
-typedef struct _ToolButtonData ToolButtonData;
+#include "commands.h"
 
-struct _ToolButtonData
-{
-  ToolType type;
-  gpointer extra_data;
-  gpointer user_data; /* Used by create_object_tool */
-};
+#include "dia_dirs.h"
 
-struct _ToolButton
-{
-  gchar **icon_data;
-  char  *tool_desc;
-  ToolButtonData callback_data;
-};
-
-static ToolButton tool_data[] =
+ToolButton tool_data[] =
 {
   { (char **) arrow_xpm,
     N_("Modify object(s)"),
+    N_("Modify"),
     { MODIFY_TOOL, NULL, NULL}
   },
   { (char **) magnify_xpm,
+    N_("Magnify"),
     N_("Magnify"),
     { MAGNIFY_TOOL, NULL, NULL}
   },
   { (char **) scroll_xpm,
     N_("Scroll around the diagram"),
+    N_("Scroll"),
     { SCROLL_TOOL, NULL, NULL}
   },
   { NULL,
     N_("Create Text"),
+    N_("Text"),
     { CREATE_OBJECT_TOOL, "Standard - Text", NULL }
   },
   { NULL,
     N_("Create Box"),
+    N_("Box"),
     { CREATE_OBJECT_TOOL, "Standard - Box", NULL }
   },
   { NULL,
     N_("Create Ellipse"),
+    N_("Ellipse"),
     { CREATE_OBJECT_TOOL, "Standard - Ellipse", NULL }
   },
+  { NULL,
+    N_("Create Polygon"),
+    N_("Polygon"),
+    { CREATE_OBJECT_TOOL, "Standard - Polygon", NULL }
+  },
   { NULL,
     N_("Create Line"),
+    N_("Line"),
     { CREATE_OBJECT_TOOL, "Standard - Line", NULL }
   },
   { NULL,
     N_("Create Arc"),
+    N_("Arc"),
     { CREATE_OBJECT_TOOL, "Standard - Arc", NULL }
   },
   { NULL,
     N_("Create Zigzagline"),
+    N_("Zigzagline"),
     { CREATE_OBJECT_TOOL, "Standard - ZigZagLine", NULL }
   },
   { NULL,
     N_("Create Polyline"),
+    N_("Polyline"),
     { CREATE_OBJECT_TOOL, "Standard - PolyLine", NULL }
   },
   { NULL,
     N_("Create Bezierline"),
+    N_("Bezierline"),
     { CREATE_OBJECT_TOOL, "Standard - BezierLine", NULL }
   },
   { NULL,
     N_("Create Image"),
+    N_("Image"),
     { CREATE_OBJECT_TOOL, "Standard - Image", NULL }
   }
 };
 
 #define NUM_TOOLS (sizeof (tool_data) / sizeof (ToolButton))
+const int num_tools = NUM_TOOLS;
+
 #define COLUMNS   4
 #define ROWS      3
 
@@ -122,6 +111,14 @@ static GSList *tool_group = NULL;
 
 GtkWidget *modify_tool_button;
 
+/*  If we create *all* menus at startup, we can read a .menurc file and
+ *  get persistent menu shortcuts.  Yummy!
+ */
+void
+create_menus() {
+}
+
+
 /*  The popup shell is a pointer to the display shell that posted the latest
  *  popup menu.  When this is null, and a command is invoked, then the
  *  assumption is that the command was a result of a keyboard accelerator
@@ -131,7 +128,7 @@ GtkWidget *popup_shell = NULL;
 void
 create_display_shell(DDisplay *ddisp,
 		     int width, int height,
-		     char *title)
+		     char *title, int top_level_window)
 {
   GtkWidget *table;
   GtkWidget *status_hbox;
@@ -149,12 +146,18 @@ create_display_shell(DDisplay *ddisp,
   ddisp->vsbdata = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, height, 1, 1, height-1));
 
   /*  The toplevel shell */
-  ddisp->shell = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (ddisp->shell), title);
-  gtk_window_set_wmclass (GTK_WINDOW (ddisp->shell), "diagram_window",
-			  "Dia");
-  gtk_window_set_policy (GTK_WINDOW (ddisp->shell), TRUE, TRUE, TRUE);
+  if (top_level_window) {
+    ddisp->shell = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_title (GTK_WINDOW (ddisp->shell), title);
+    gtk_window_set_wmclass (GTK_WINDOW (ddisp->shell), "diagram_window",
+			    "Dia");
+    gtk_window_set_policy (GTK_WINDOW (ddisp->shell), TRUE, TRUE, TRUE);
+  } else {
+    ddisp->shell = gtk_event_box_new ();
+  }
+  
   gtk_object_set_user_data (GTK_OBJECT (ddisp->shell), (gpointer) ddisp);
+
   gtk_widget_set_events (ddisp->shell,
 			 GDK_POINTER_MOTION_MASK |
 			 GDK_POINTER_MOTION_HINT_MASK |
@@ -168,6 +171,12 @@ create_display_shell(DDisplay *ddisp,
                       GTK_SIGNAL_FUNC (ddisplay_destroy),
                       ddisp);
 
+  /* Clipboard handling signals */
+  gtk_signal_connect (GTK_OBJECT(ddisp->shell), "selection_get",
+		      GTK_SIGNAL_FUNC (get_selection_handler), NULL);
+  gtk_signal_connect (GTK_OBJECT(ddisp->shell), "selection_received",
+		      GTK_SIGNAL_FUNC (received_selection_handler), NULL);
+
   /*  the table containing all widgets  */
   table = gtk_table_new (4, 3, FALSE);
   gtk_table_set_col_spacing (GTK_TABLE (table), 0, 1);
@@ -236,8 +245,18 @@ create_display_shell(DDisplay *ddisp,
   menus_get_image_menu (&ddisp->popup, &ddisp->accel_group);
 #endif
 
-  /*  the accelerator table/group for the popup */
-  gtk_window_add_accel_group (GTK_WINDOW(ddisp->shell), ddisp->accel_group);
+  if (top_level_window) {
+    gchar *accelfilename = dia_config_filename("menus/display");
+
+    /*  the accelerator table/group for the popup */
+    gtk_window_add_accel_group (GTK_WINDOW(ddisp->shell),
+				ddisp->accel_group);
+    
+    if (accelfilename) {
+      gtk_item_factory_parse_rc(accelfilename);
+      g_free(accelfilename);
+    }
+  }
 
 
   /* the statusbars */
@@ -272,14 +291,20 @@ create_display_shell(DDisplay *ddisp,
   gtk_widget_grab_focus (ddisp->canvas);
 }
 
-static void
+void
 tool_select_update (GtkWidget *w,
 		     gpointer   data)
 {
   ToolButtonData *tooldata = (ToolButtonData *) data;
 
-  if ((tooldata->type != -1) && GTK_TOGGLE_BUTTON (w)->active)
+  if (tooldata == NULL) {
+    g_warning(_("NULL tooldata in tool_select_update"));
+    return;
+  }
+
+  if (tooldata->type != -1) {
     tool_select (tooldata->type, tooldata->extra_data, tooldata->user_data);
+  }
 }
 
 static gint
@@ -298,6 +323,21 @@ tool_button_press (GtkWidget      *w,
   return FALSE;
 }
 
+void 
+tool_select_callback(GtkWidget *widget, gpointer data) {
+  ToolButtonData *tooldata = (ToolButtonData *)data;
+
+  if (tooldata == NULL) {
+    g_warning("NULL tooldata in tool_select_callback");
+    return;
+  }
+
+  if (tooldata->type != -1) {
+    tool_select (tooldata->type, tooldata->extra_data, tooldata->user_data);
+    
+  }
+}
+
 static void
 create_tools(GtkWidget *parent)
 {
@@ -309,6 +349,7 @@ create_tools(GtkWidget *parent)
   GdkPixmap *pixmap;
   GdkBitmap *mask;
   GtkStyle *style;
+
   char **pixmap_data;
   int i;
 
@@ -325,7 +366,11 @@ create_tools(GtkWidget *parent)
 
 #ifdef USE_WRAPBOX
     gtk_wrap_box_pack(GTK_WRAP_BOX(parent), button,
+# if 1 /* HB: IHMO even 13 buttons look better when stretched to parent */
 		      TRUE, TRUE, FALSE, TRUE);
+# else
+		      FALSE, TRUE, FALSE, TRUE);
+# endif
 #else
     gtk_table_attach (GTK_TABLE (table), button,
 		      (i % COLUMNS), (i % COLUMNS) + 1,
@@ -369,7 +414,9 @@ create_tools(GtkWidget *parent)
     gtk_signal_connect (GTK_OBJECT (button), "button_press_event",
 			GTK_SIGNAL_FUNC (tool_button_press),
 			&tool_data[i].callback_data);
-    
+
+    tool_data[i].callback_data.widget = button;
+
     gtk_tooltips_set_tip (tool_tips, button,
 			  gettext(tool_data[i].tool_desc), NULL);
     
@@ -453,6 +500,8 @@ create_sheet_page(GtkWidget *parent, Sheet *sheet)
 #ifdef USE_WRAPBOX
     gtk_wrap_box_pack(GTK_WRAP_BOX(table), button,
 		      FALSE, TRUE, FALSE, TRUE);
+    if (sheet_obj->line_break)
+      gtk_wrap_box_set_child_forced_break(GTK_WRAP_BOX(table),button,TRUE);
 #else
     gtk_table_attach (GTK_TABLE (table), button,
 		      (i % COLUMNS), (i % COLUMNS) + 1,
@@ -684,13 +733,14 @@ create_lineprops_area(GtkWidget *parent)
   gtk_widget_show(chooser);
 }
 
-void
+static void
 toolbox_delete (GtkWidget *widget, gpointer data)
 {
-  app_exit();
+  if (!app_is_embedded())
+    app_exit();
 }
 
-void
+static void
 toolbox_destroy (GtkWidget *widget, gpointer data)
 {
   app_exit();
@@ -702,6 +752,7 @@ create_toolbox ()
   GtkWidget *window;
   GtkWidget *main_vbox;
   GtkWidget *vbox;
+  gchar *accelfilename;
 #ifndef GNOME
   GtkWidget *menubar;
   GtkAccelGroup *accel_group;
@@ -744,6 +795,12 @@ create_toolbox ()
   gtk_widget_show (menubar);
 #endif
   
+  accelfilename = dia_config_filename("menus/toolbox");
+  
+  if (accelfilename) {
+    gtk_item_factory_parse_rc(accelfilename);
+    g_free(accelfilename);
+  }
   /*  tooltips  */
   tool_tips = gtk_tooltips_new ();
 
@@ -768,7 +825,17 @@ create_toolbox ()
   create_color_area (vbox);
   create_lineprops_area (vbox);
   
-  gtk_widget_show (window);
   toolbox_shell = window;
 }
 
+void
+toolbox_show(void)
+{
+  gtk_widget_show(toolbox_shell);
+}
+
+void
+toolbox_hide(void)
+{
+  gtk_widget_hide(toolbox_shell);
+}
