@@ -24,11 +24,14 @@
 #endif
 #include "intl.h"
 #include "menus.h"
+#include "tool.h"
 #include "commands.h"
 #include "message.h"
-/*#include "interface.h"*/
+#include "interface.h"
 #include "display.h"
 #include "filedlg.h"
+#include "plugin-manager.h"
+#include "select.h"
 
 #if GNOME
 static GnomeUIInfo toolbox_filemenu[] = {
@@ -36,6 +39,7 @@ static GnomeUIInfo toolbox_filemenu[] = {
 			    file_new_callback, NULL),
   GNOMEUIINFO_MENU_OPEN_ITEM(file_open_callback, NULL),
   GNOMEUIINFO_MENU_PREFERENCES_ITEM(file_preferences_callback, NULL),
+  GNOMEUIINFO_ITEM_NONE(N_("Plug-ins"), NULL, file_plugins_callback),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_EXIT_ITEM(file_quit_callback, NULL),
   GNOMEUIINFO_END  
@@ -76,6 +80,9 @@ static GnomeUIInfo editmenu[] = {
     GDK_D, GDK_CONTROL_MASK },
   GNOMEUIINFO_MENU_UNDO_ITEM(edit_undo_callback, NULL),
   GNOMEUIINFO_MENU_REDO_ITEM(edit_redo_callback, NULL),
+  GNOMEUIINFO_ITEM_NONE(N_("Copy Text"), NULL, edit_copy_text_callback),
+  GNOMEUIINFO_ITEM_NONE(N_("Cut Text"), NULL, edit_cut_text_callback),
+  GNOMEUIINFO_ITEM_NONE(N_("Paste _Text"), NULL, edit_paste_text_callback),
   GNOMEUIINFO_END
 };
 
@@ -120,21 +127,52 @@ static GnomeUIInfo viewmenu[] = {
   GNOMEUIINFO_END
 };
 
+static GnomeUIInfo selectmenu[] = {
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("All"), NULL, select_all_callback, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("None"), NULL, select_none_callback, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Invert"), NULL, select_invert_callback, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Connected"), NULL, select_connected_callback, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Transitive"), NULL, select_transitive_callback, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Same Type"), NULL, select_same_type_callback, 0),
+  GNOMEUIINFO_SEPARATOR,
+  GNOMEUIINFO_TOGGLEITEM(N_("Replace"), NULL, select_style_callback, 
+			 GINT_TO_POINTER(0)),
+  GNOMEUIINFO_TOGGLEITEM(N_("Union"), NULL, select_style_callback,
+			 GINT_TO_POINTER(1)),
+  GNOMEUIINFO_TOGGLEITEM(N_("Intersect"), NULL, select_style_callback,
+			 GINT_TO_POINTER(2)),
+  GNOMEUIINFO_TOGGLEITEM(N_("Remove"), NULL, select_style_callback, 
+			 GINT_TO_POINTER(3)),
+  GNOMEUIINFO_TOGGLEITEM(N_("Invert"), NULL, select_style_callback, 
+			 GINT_TO_POINTER(4)),
+  GNOMEUIINFO_END
+};
+
 static GnomeUIInfo objects_align_h[] = {
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Left"), NULL, objects_align_h_callback, 0),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Center"), NULL, objects_align_h_callback, 1),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Right"), NULL, objects_align_h_callback, 2),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Equal Distance"), NULL, objects_align_h_callback, 4),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Adjacent"), NULL, objects_align_h_callback, 5),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Left"), NULL, objects_align_h_callback,
+			     GINT_TO_POINTER(0)),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Center"), NULL, objects_align_h_callback,
+			     GINT_TO_POINTER(1)),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Right"), NULL, objects_align_h_callback,
+			     GINT_TO_POINTER(2)),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Equal Distance"), NULL, objects_align_h_callback,
+			     GINT_TO_POINTER(4)),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Adjacent"), NULL, objects_align_h_callback,
+			     GINT_TO_POINTER(5)),
   GNOMEUIINFO_END
 };
 
 static GnomeUIInfo objects_align_v[] = {
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Top"), NULL, objects_align_v_callback, 0),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Center"), NULL, objects_align_v_callback, 1),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Bottom"), NULL, objects_align_v_callback, 2),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Equal Distance"), NULL, objects_align_v_callback, 4),
-  GNOMEUIINFO_ITEM_NONE_DATA(N_("Adjacent"), NULL, objects_align_h_callback, 5),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Top"), NULL, objects_align_v_callback,
+			     GINT_TO_POINTER(0)),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Center"), NULL, objects_align_v_callback,
+			     GINT_TO_POINTER(1)),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Bottom"), NULL, objects_align_v_callback,
+			     GINT_TO_POINTER(2)),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Equal Distance"), NULL, objects_align_v_callback,
+			     GINT_TO_POINTER(4)),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Adjacent"), NULL, objects_align_h_callback,
+			     GINT_TO_POINTER(5)),
   GNOMEUIINFO_END
 };
 
@@ -150,6 +188,23 @@ static GnomeUIInfo objectsmenu[] = {
   GNOMEUIINFO_END
 };
 
+static GnomeUIInfo toolsmenu[] = {
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Modify"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Magnify"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Scroll"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Text"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Box"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Ellipse"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Polygon"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Line"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Arc"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Zigzagline"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Polyline"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Bezierline"), NULL, NULL, 0),
+  GNOMEUIINFO_ITEM_NONE_DATA(N_("Image"), NULL, NULL, 0),
+  GNOMEUIINFO_END
+};
+
 static GnomeUIInfo dialogsmenu[] = {
   GNOMEUIINFO_ITEM_NONE(N_("_Properties"), NULL, dialogs_properties_callback),
   GNOMEUIINFO_ITEM_NONE(N_("_Layers"), NULL, dialogs_layers_callback),
@@ -171,11 +226,17 @@ static GnomeUIInfo display_menu[] = {
   GNOMEUIINFO_MENU_FILE_TREE(filemenu),
   GNOMEUIINFO_MENU_EDIT_TREE(editmenu),
   GNOMEUIINFO_MENU_VIEW_TREE(viewmenu),
+  GNOMEUIINFO_SUBTREE(N_("Select"), selectmenu),
   GNOMEUIINFO_SUBTREE(N_("Objects"), objectsmenu),
+  GNOMEUIINFO_SUBTREE(N_("Tools"), toolsmenu),
   GNOMEUIINFO_SUBTREE(N_("Dialogs"), dialogsmenu),
   GNOMEUIINFO_END
 };
-#endif
+
+GtkWidget *toolbox_menubar = NULL;
+GtkWidget *display_menubar = NULL;
+
+#else /* !GNOME */
 
 static GtkItemFactoryEntry toolbox_menu_items[] =
 {
@@ -183,6 +244,7 @@ static GtkItemFactoryEntry toolbox_menu_items[] =
   {N_("/File/_New"),           "<control>N", file_new_callback,         0 },
   {N_("/File/_Open"),          "<control>O", file_open_callback,        0 },
   {N_("/File/_Preferences..."),NULL,         file_preferences_callback, 0 },
+  {N_("/File/P_lugins"),       NULL,         file_plugins_callback,     0 },
   {N_("/File/sep1"),           NULL,         NULL,                      0, "<Separator>"},
   {N_("/File/_Quit"),          "<control>Q", file_quit_callback,        0 },
   {N_("/_Help"),               NULL,         NULL,                      0, "<Branch>" },
@@ -210,8 +272,11 @@ static GtkItemFactoryEntry display_menu_items[] =
   {N_("/Edit/C_ut"),              "<control>X", edit_cut_callback,          0},
   {N_("/Edit/_Paste"),            "<control>V", edit_paste_callback,        0},
   {N_("/Edit/_Delete"),           "<control>D", edit_delete_callback,       0},
-  {N_("/Edit/_Undo"),             "<control>Z", edit_undo_callback,        0},
-  {N_("/Edit/_Redo"),             "<control>R", edit_redo_callback,        0},
+  {N_("/Edit/_Undo"),             "<control>Z", edit_undo_callback,         0},
+  {N_("/Edit/_Redo"),             "<control>R", edit_redo_callback,         0},
+  {N_("/Edit/Copy Text"),         NULL,         edit_copy_text_callback,    0},
+  {N_("/Edit/Cut Text"),          NULL,         edit_cut_text_callback,     0},
+  {N_("/Edit/Paste _Text"),       NULL,         edit_paste_text_callback,   0},
   {N_("/_View"),                  NULL,         NULL,                       0, "<Branch>"},
   /*  {"/View/tearoff1 ",         NULL,         tearoff,                     0, "<Tearoff>" }, */
   {N_("/View/Zoom _In"),          NULL,          view_zoom_in_callback,     0},
@@ -238,6 +303,18 @@ static GtkItemFactoryEntry display_menu_items[] =
   {N_("/View/sep1"),              NULL,         NULL,                       0, "<Separator>"},
   {N_("/View/New _View"),         "<control>I", view_new_view_callback,     0},
   {N_("/View/Show _All"),         "<control>A", view_show_all_callback,     0},
+  {N_("/Select/All"), NULL, select_all_callback, 0},
+  {N_("/Select/None"), NULL, select_none_callback, 0},
+  {N_("/Select/Invert"), NULL, select_invert_callback, 0},
+  {N_("/Select/Connected"), NULL, select_connected_callback, 0},
+  {N_("/Select/Transitive"), NULL, select_transitive_callback, 0},
+  {N_("/Select/Same Type"), NULL, select_same_type_callback, 0},
+  {N_("/Select/sep1"),           NULL,         NULL,                       0, "<Separator>"},
+  {N_("/Select/Replace"), NULL, select_style_callback, 0, "<CheckItem>"},
+  {N_("/Select/Union"), NULL, select_style_callback, 1, "<CheckItem>"},
+  {N_("/Select/Intersect"), NULL, select_style_callback, 2, "<CheckItem>"},
+  {N_("/Select/Remove"), NULL, select_style_callback, 3, "<CheckItem>"},
+  {N_("/Select/Invert"), NULL, select_style_callback, 4, "<CheckItem>"},
   {N_("/_Objects"),               NULL,         NULL,                       0, "<Branch>"},
   /*  {"/Objects/tearoff1 ",      NULL,         tearoff,                    0, "<Tearoff>" }, */
   {N_("/Objects/Send to _Back"),  "<control>B", objects_place_under_callback,0},
@@ -259,6 +336,20 @@ static GtkItemFactoryEntry display_menu_items[] =
   {N_("/Objects/Align Vertical/Equal Distance"),   NULL, objects_align_v_callback,    4},
   {N_("/Objects/Align Vertical/Adjacent"),   NULL, objects_align_v_callback,    5},
 
+  {N_("/Tools/Modify"), NULL, NULL, 0},
+  {N_("/Tools/Magnify"), NULL, NULL, 0},
+  {N_("/Tools/Scroll"), NULL, NULL, 0},
+  {N_("/Tools/Text"), NULL, NULL, 0},
+  {N_("/Tools/Box"), NULL, NULL, 0},
+  {N_("/Tools/Ellipse"), NULL, NULL, 0},
+  {N_("/Tools/Polygon"), NULL, NULL, 0},
+  {N_("/Tools/Line"), NULL, NULL, 0},
+  {N_("/Tools/Arc"), NULL, NULL, 0},
+  {N_("/Tools/Zigzagline"), NULL, NULL, 0},
+  {N_("/Tools/Polyline"), NULL, NULL, 0},
+  {N_("/Tools/Bezierline"), NULL, NULL, 0},
+  {N_("/Tools/Image"), NULL, NULL, 0},
+
   /*  {"/Objects/tearoff1 ",      NULL,         tearoff,                     0, "<Tearoff>" }, */
   {N_("/_Dialogs"),               NULL,         NULL,                       0, "<Branch>"},
   {N_("/Dialogs/_Properties"),    NULL,         dialogs_properties_callback,0},
@@ -269,21 +360,60 @@ static GtkItemFactoryEntry display_menu_items[] =
 static int toolbox_nmenu_items = sizeof(toolbox_menu_items) / sizeof(toolbox_menu_items[0]);
 static int display_nmenu_items = sizeof(display_menu_items) / sizeof(display_menu_items[0]);
 
+#endif /* !GNOME */
+
 static GtkItemFactory *toolbox_item_factory = NULL;
 static GtkItemFactory *display_item_factory = NULL;
 static GtkAccelGroup *display_accel_group = NULL;
 
+static void
+tool_menu_select(GtkWidget *w, gpointer   data) {
+  ToolButtonData *tooldata = (ToolButtonData *) data;
+
+  if (tooldata == NULL) {
+    g_warning(_("NULL tooldata in tool_menu_select"));
+    return;
+  }
+
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tooldata->widget),TRUE);
+}
+
 #ifdef GNOME
 void
 gnome_toolbox_menus_create(GtkWidget* app)
 {
   gnome_app_create_menus(GNOME_APP(app), toolbox_menu);
+  toolbox_menubar = GNOME_APP(app)->menubar;
 }
 
 GtkWidget *
 gnome_display_menus_create()
 {
-  return gnome_popup_menu_new(display_menu);
+  GtkWidget *new_menu;
+  GtkMenuItem *menuitem;
+  GString *path;
+  char *display = "<Display>";
+  int i;
+
+  new_menu = gnome_popup_menu_new(display_menu);
+
+  path = g_string_new ("<Display>");
+
+  /* There may be a slightly faster way to do this */
+
+  for (i = 0; i < num_tools; i++) {
+    g_string_append (g_string_assign(path, display),_("/Tools/"));
+    g_string_append (path, tool_data[i].menuitem_name);
+    menuitem = (GtkMenuItem *)menus_get_item_from_path(path->str);
+
+    if (menuitem != NULL) {
+      gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
+                          GTK_SIGNAL_FUNC (tool_menu_select),
+                          &tool_data[i].callback_data);
+    }    
+  }
+
+  return new_menu;
 }
 #endif
 
@@ -297,6 +427,7 @@ tearoff (gpointer             callback_data,
 }
 #endif
 
+#ifndef GNOME
 static GtkItemFactoryEntry *
 translate_entries (const GtkItemFactoryEntry *entries, gint n)
 {
@@ -344,11 +475,17 @@ menus_get_toolbox_menubar (GtkWidget **menubar,
 
   *menubar = gtk_item_factory_get_widget (toolbox_item_factory, "<Toolbox>");
 }
+
 void
 menus_get_image_menu (GtkWidget **menu,
 		      GtkAccelGroup **accel_group)
 {
+  GtkWidget *new_menu;
+  GtkMenuItem *menuitem;
+  GString *path;
+  char *display = "<Display>";
   GtkItemFactoryEntry *translated_entries;
+  int i;
 
   if (display_item_factory == NULL) {
     display_accel_group = gtk_accel_group_new ();
@@ -364,30 +501,53 @@ menus_get_image_menu (GtkWidget **menu,
   
   *accel_group = display_accel_group;
   *menu = gtk_item_factory_get_widget (display_item_factory, "<Display>");
-}
 
+  path = g_string_new ("<Display>");
+
+  /* There may be a slightly faster way to do this */
+
+  for (i = 0; i < num_tools; i++) {
+    g_string_append (g_string_assign(path, display),_("/Tools/"));
+    g_string_append (path, tool_data[i].menuitem_name);
+    menuitem = (GtkMenuItem *)menus_get_item_from_path(path->str);
+
+    if (menuitem != NULL) {
+      gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
+                          GTK_SIGNAL_FUNC (tool_menu_select),
+                          &tool_data[i].callback_data);
+    }    
+  }
+}
+#endif /* !GNOME */
 
 GtkWidget *menus_get_item_from_path (char *path)
 {
-  GtkWidget *widget;
+  GtkWidget *widget = NULL;
 
 # ifdef GNOME
   gint pos;
-  GtkWidget *parentw;
+  char *menu_name;
+  GtkWidget *parentw = NULL;
   GtkMenuShell *parent;
-  DDisplay *ddisp = ddisplay_active ();
+  DDisplay *ddisp;
 
-  if (! ddisp) return NULL;
-
-  /* drop the <Display>/ at the start */
+  /* drop the "<Display>/" or "<Toolbox>/" at the start */
+  menu_name = path;
   if (! (path = strchr (path, '/'))) return NULL;
   path ++; /* move past the / */
 
-  parentw = gnome_app_find_menu_pos (ddisp->popup, path, &pos);
+  if (strncmp(menu_name, "<Display>", strlen("<Display>")) == 0) {
+    ddisp = ddisplay_active ();
+    if (! ddisp)
+      return NULL;
+    
+    parentw = gnome_app_find_menu_pos(ddisp->popup, path, &pos);
+  } else if (strncmp(menu_name, "<Toolbox>", strlen("<Toolbox>")) == 0) {
+    parentw = gnome_app_find_menu_pos(toolbox_menubar, path, &pos);
+  } 
   if (! parentw) {
     g_warning("Can't find menu entry '%s'!\nThis is probably a i18n problem "
-	      "(try LANG=C).",
-	      path);
+	      "(try LANG=C).", path);
     return NULL;
   }
 
@@ -396,14 +556,19 @@ GtkWidget *menus_get_item_from_path (char *path)
 
 # else
 
-  widget = gtk_item_factory_get_widget(display_item_factory, path);
-  if (widget == NULL)
+  if (display_item_factory) {
+    widget = gtk_item_factory_get_widget(display_item_factory, path);
+  }
+  
+  if (widget == NULL) {
     widget = gtk_item_factory_get_widget(toolbox_item_factory, path);
+  }
 
-  if (widget == NULL)
+  if (widget == NULL) {
      g_warning("Can't find menu entry '%s'!\nThis is probably a i18n problem "
 	       "(try LANG=C).",
 	       path);
+  }
 
 # endif
 
