@@ -116,7 +116,9 @@ paginate_psprint(Diagram *dia, FILE *file)
   gfloat width, height;
   gfloat x, y, initx, inity;
   guint nobjs = 0;
+  char *old_locale;
 
+  old_locale = setlocale(LC_NUMERIC, "C");
   rend = new_psprint_renderer(dia, file);
 
   /* the usable area of the page */
@@ -147,6 +149,8 @@ paginate_psprint(Diagram *dia, FILE *file)
     }
 
   destroy_eps_renderer(rend);
+
+  setlocale(LC_NUMERIC, old_locale);
 }
 
 static void
@@ -170,10 +174,14 @@ diagram_print_ps(Diagram *dia)
   GtkWidget *iscmd, *isofile;
   GtkWidget *cmd, *ofile;
   gboolean cont = FALSE;
-  
+  DDisplay *ddisp;
+
   FILE *file;
   gboolean is_pipe;
 
+  ddisp = ddisplay_active();
+  dia = ddisp->diagram;
+
   /* create the dialog */
   dialog = gtk_dialog_new();
   gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
@@ -241,6 +249,7 @@ diagram_print_ps(Diagram *dia)
   gtk_entry_set_text(GTK_ENTRY(cmd), 
 		     last_print_options.command 
 		     ? last_print_options.command : "lpr");
+  /* Ought to use filename+extension here */
   gtk_entry_set_text(GTK_ENTRY(ofile), 
 		     last_print_options.output 
 		     ? last_print_options.output : "output.ps");
@@ -260,7 +269,19 @@ diagram_print_ps(Diagram *dia)
     file = popen(gtk_entry_get_text(GTK_ENTRY(cmd)), "w");
     is_pipe = TRUE;
   } else {
-    file = fopen(gtk_entry_get_text(GTK_ENTRY(ofile)), "w");
+    gchar *filename = gtk_entry_get_text(GTK_ENTRY(ofile));
+    if (filename[0] != '/') {
+      char *diagram_dir;
+      char *full_filename;
+
+      diagram_dir = g_dirname(dia->filename);
+      full_filename = g_malloc(strlen(diagram_dir)+strlen(filename)+2);
+      sprintf(full_filename, "%s/%s", diagram_dir, filename);
+      file = fopen(full_filename, "w");
+      g_free(full_filename);
+    } else {
+      file = fopen(filename, "w");
+    }
     is_pipe = FALSE;
   }
 
