@@ -27,6 +27,7 @@
 #ifdef HAVE_STDDEF_H
 #include <stddef.h>
 #endif
+#include <locale.h>
 
 #ifdef GNOME
 #  include <gnome.h>
@@ -37,7 +38,9 @@
 #include "intl.h"
 #include "widgets.h"
 #include "diagram.h"
+#include "message.h"
 #include "preferences.h"
+#include "dia_dirs.h"
 
 struct DiaPreferences prefs;
 
@@ -98,7 +101,7 @@ struct DiaPrefsData prefs_data[] =
   { "reset_tools_after_create", PREF_BOOLEAN, PREF_OFFSET(reset_tools_after_create), &default_false, 0, N_("Reset tools after create:") },
   { "compress_save", PREF_BOOLEAN, PREF_OFFSET(compress_save), &default_true, 0, N_("Compress saved files:") },
   { "undo_depth", PREF_UINT, PREF_OFFSET(undo_depth), &default_undo_depth, 0, N_("Number of undo levels:") },
-
+  { "reverse_rubberbanding_intersects", PREF_BOOLEAN, PREF_OFFSET(reverse_rubberbanding_intersects), &default_true, 0, N_("Reverse dragging selects\nintersecting objects:") },
   { NULL, PREF_NONE, 0, NULL, 1, N_("New window:") },
   { "new_view_width", PREF_UINT, PREF_OFFSET(new_view.width), &default_int_w, 1, N_("Width:") },
   { "new_view_height", PREF_UINT, PREF_OFFSET(new_view.height), &default_int_h, 1, N_("Height:") },
@@ -220,16 +223,23 @@ prefs_save(void)
 {
   int i;
   char *ptr;
-  char filename[512];
+  gchar *filename;
+  char *old_locale;
   FILE *file;
 
-  strncpy(filename, getenv("HOME"), 512);
-  filename[511] = 0;
-  strncat(filename, "/.dia/diarc",512);
-  filename[511] = 0;
-  
+  filename = dia_config_filename("diarc");
+
   file = fopen(filename, "w");
+
+  if (!file) {
+    message_error(_("Could not open `%s' for writing"), filename);
+    g_free(filename);
+    return;
+  }
   
+  g_free(filename);
+
+  old_locale = setlocale(LC_NUMERIC, "C");
   fprintf(file, "# Note: This file is automatically generated by Dia\n");
   for (i=0;i<NUM_PREFS_DATA;i++) {
     if (prefs_data[i].type == PREF_NONE)
@@ -249,6 +259,7 @@ prefs_save(void)
       break;
     case PREF_REAL:
     case PREF_UREAL:
+      
       fprintf(file, "%f\n", (double) *(real *)ptr);
       break;
     case PREF_COLOUR:
@@ -258,6 +269,7 @@ prefs_save(void)
       break;
     }
   }
+  setlocale(LC_NUMERIC, old_locale);
   fclose(file);
 }
 
@@ -337,25 +349,26 @@ void
 prefs_load(void)
 {
   int i;
-  char filename[512];
+  gchar *filename;
   int fd;
   GScanner *scanner;
   guint expected_token;
 
-  strncpy(filename, getenv("HOME"), 512);
-  filename[511] = 0;
-  strncat(filename, "/.dia/diarc",512);
-  filename[511] = 0;
-  
+  filename = dia_config_filename("diarc");
+
   fd = open(filename, O_RDONLY);
 
   if (fd < 0) {
-    strncpy(filename, getenv("HOME"), 512);
-    filename[511] = 0;
-    strncat(filename, "/.diarc",512);
-    filename[511] = 0;
+    char *homedir = getenv("HOME");
+
+    g_free(filename);
+
+    filename = g_malloc(strlen(homedir)+strlen("/.diarc")+1);
+    strcpy(filename, getenv("HOME"));
+    strcat(filename, "/.diarc");
     fd = open(filename, O_RDONLY);
   }
+  g_free(filename);
 
   prefs_set_defaults();
 
