@@ -18,10 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <math.h>
 #include <unistd.h>
+#include <locale.h>
 
 #include <entities.h>
 
@@ -772,10 +774,14 @@ static void
 export_svg(DiagramData *data, const gchar *filename, const gchar *diafilename)
 {
   RendererSVG *renderer;
+  char *old_locale;
 
-  renderer = new_svg_renderer(data, filename);
-  data_render(data, (Renderer *)renderer, NULL, NULL, NULL);
-  destroy_svg_renderer(renderer);
+  old_locale = setlocale(LC_NUMERIC, "C");
+  if ((renderer = new_svg_renderer(data, filename))) {
+    data_render(data, (Renderer *)renderer, NULL, NULL, NULL);
+    destroy_svg_renderer(renderer);
+  }
+  setlocale(LC_NUMERIC, old_locale);
 }
 
 static const gchar *extensions[] = { "svg", NULL };
