@@ -31,7 +31,10 @@ typedef enum {
   ARROW_FILLED_TRIANGLE,
   ARROW_HOLLOW_DIAMOND,
   ARROW_FILLED_DIAMOND,
-  ARROW_HALF_HEAD
+  ARROW_HALF_HEAD,
+  ARROW_SLASHED_CROSS,
+  ARROW_FILLED_ELLIPSE,
+  ARROW_HOLLOW_ELLIPSE
 } ArrowType;
 
 typedef struct {
