@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,8 @@
 
 #include "geometry.h"
 #include "color.h"
+#include "arrows.h"
+#include "render.h"
 
 extern Color attributes_get_foreground(void);
 extern Color attributes_get_background(void);
@@ -31,4 +33,15 @@ extern void attributes_default_fgbg(void);
 extern real attributes_get_default_linewidth(void);
 extern void attributes_set_default_linewidth(real width);
 
+extern Arrow attributes_get_default_start_arrow(void);
+extern void attributes_set_default_start_arrow(Arrow arrow);
+
+extern Arrow attributes_get_default_end_arrow(void);
+extern void attributes_set_default_end_arrow(Arrow arrow);
+
+extern void attributes_get_default_line_style(LineStyle *style,
+					      real *dash_length);
+extern void attributes_set_default_line_style(LineStyle style,
+					      real dash_length);
+
 #endif /* ATTRIBUTES_H */
