@@ -24,6 +24,7 @@
 #include <math.h>
 
 #include "bezier_conn.h"
+#include "intl.h"
 #include "message.h"
 
 #define HANDLE_BEZMAJOR  (HANDLE_CUSTOM1)
@@ -42,6 +43,7 @@ struct PointChange {
   int applied;
   
   BezPoint point;
+  BezCornerType corner_type;
   int pos;
   /* owning ref when not applied for ADD_POINT
    * owning ref when applied for REMOVE_POINT */
@@ -50,12 +52,30 @@ struct PointChange {
   ConnectionPoint *connected_to1, *connected_to2, *connected_to3;
 };
 
+struct CornerChange {
+  ObjectChange obj_change;
+  /* Only one kind of corner_change */
+  int applied;
+
+  Handle *handle;
+  /* Old places when SET_CORNER_TYPE is applied */
+  Point point_left, point_right;
+  BezCornerType old_type, new_type;
+};
+
 static ObjectChange *
-bezierconn_create_change(BezierConn *bez, enum change_type type,
-			 BezPoint *point, int segment,
-			 Handle *handle1, ConnectionPoint *connected_to1,
-			 Handle *handle2, ConnectionPoint *connected_to2,
-			 Handle *handle3, ConnectionPoint *connected_to3);
+bezierconn_create_point_change(BezierConn *bez, enum change_type type,
+			       BezPoint *point, BezCornerType corner_type,
+			       int segment,
+			       Handle *handle1, ConnectionPoint *connected_to1,
+			       Handle *handle2, ConnectionPoint *connected_to2,
+			       Handle *handle3, ConnectionPoint *connected_to3);
+static ObjectChange *
+bezierconn_create_corner_change(BezierConn *bez, Handle *handle,
+				Point *point_left, Point *point_right,
+				BezCornerType old_corner_type,
+				BezCornerType new_corner_type);
+
 
 static void setup_corner_handle(Handle *handle, HandleId id)
 {
@@ -76,7 +96,8 @@ static int get_handle_nr(BezierConn *bez, Handle *handle)
   return -1;
 }
 
-#define get_comp_nr(hnum) (((int)hnum+2)/3)
+#define get_comp_nr(hnum) (((int)(hnum)+2)/3)
+#define get_major_nr(hnum) (((int)(hnum)+1)/3)
 
 void
 bezierconn_move_handle(BezierConn *bez, Handle *handle,
@@ -110,19 +131,63 @@ bezierconn_move_handle(BezierConn *bez, Handle *handle,
   case HANDLE_LEFTCTRL:
     bez->points[comp_nr].p2 = *to;
     if (comp_nr < bez->numpoints - 1) {
-      pt = bez->points[comp_nr].p3;
-      point_sub(&pt, &bez->points[comp_nr].p2);
-      point_add(&pt, &bez->points[comp_nr].p3);
-      bez->points[comp_nr+1].p1 = pt;
+      switch (bez->corner_types[comp_nr]) {
+      case BEZ_CORNER_SYMMETRIC:
+	pt = bez->points[comp_nr].p3;
+	point_sub(&pt, &bez->points[comp_nr].p2);
+	point_add(&pt, &bez->points[comp_nr].p3);
+	bez->points[comp_nr+1].p1 = pt;
+	break;
+      case BEZ_CORNER_SMOOTH: {
+	real len;
+	pt = bez->points[comp_nr+1].p1;
+	point_sub(&pt, &bez->points[comp_nr].p3);
+	len = point_len(&pt);
+	pt = bez->points[comp_nr].p2;
+	point_sub(&pt, &bez->points[comp_nr].p3);
+	if (point_len(&pt) > 0)
+	  point_normalize(&pt);
+	else { pt.x = 1.0; pt.y = 0.0; }	  
+	point_scale(&pt, -len);
+	point_add(&pt, &bez->points[comp_nr].p3);
+	bez->points[comp_nr+1].p1 = pt;
+	break;
+      }	
+      case BEZ_CORNER_CUSP:
+	/* Do nothing to the other guy */
+	break;
+      }
     }
     break;
   case HANDLE_RIGHTCTRL:
     bez->points[comp_nr].p1 = *to;
     if (comp_nr > 1) {
-      pt = bez->points[comp_nr - 1].p3;
-      point_sub(&pt, &bez->points[comp_nr].p1);
-      point_add(&pt, &bez->points[comp_nr - 1].p3);
-      bez->points[comp_nr-1].p2 = pt;
+      switch (bez->corner_types[comp_nr-1]) {
+      case BEZ_CORNER_SYMMETRIC:
+	pt = bez->points[comp_nr - 1].p3;
+	point_sub(&pt, &bez->points[comp_nr].p1);
+	point_add(&pt, &bez->points[comp_nr - 1].p3);
+	bez->points[comp_nr-1].p2 = pt;
+	break;
+      case BEZ_CORNER_SMOOTH: {
+	real len;
+	pt = bez->points[comp_nr-1].p2;
+	point_sub(&pt, &bez->points[comp_nr-1].p3);
+	len = point_len(&pt);
+	pt = bez->points[comp_nr].p1;
+	point_sub(&pt, &bez->points[comp_nr-1].p3);
+	if (point_len(&pt) > 0)
+	  point_normalize(&pt);
+	else { pt.x = 1.0; pt.y = 0.0; }	  
+	point_scale(&pt, -len);
+	point_add(&pt, &bez->points[comp_nr-1].p3);
+	bez->points[comp_nr-1].p2 = pt;
+	break;
+      }	
+      case BEZ_CORNER_CUSP:
+	/* Do nothing to the other guy */
+	break;
+      }
     }
     break;
   default:
@@ -148,79 +213,25 @@ bezierconn_move(BezierConn *bez, Point *to)
   }
 }
 
-/* use a NSEGS line segment approximation to deduce the distance to a
- * segment */
-
-#define NSEGS 5
-
-static real
-segment_distance(BezierConn *bez, Point *point, real line_width, int segnum)
-{
-  static real coeff[NSEGS+1][4];
-  static gboolean coeff_valid = FALSE;
-  int i;
-  real dist = G_MAXDOUBLE;
-  Point pt, prev;
-
-  /* only calculate these coefficients once -- slight speed increase */
-  if (!coeff_valid) {
-    for (i = 0; i <= NSEGS; i++) {
-      real t1 = ((real)i)/NSEGS, t2 = t1*t1, t3 = t1*t2;
-      real it1 = 1-t1, it2 = it1*it1, it3 = it1*it2;
-
-      coeff[i][0] = it3;
-      coeff[i][1] = 3 * t1 * it2;
-      coeff[i][2] = 3 * t2 * it1;
-      coeff[i][3] = t3;
-    }
-  }
-  coeff_valid = TRUE;
-
-  if (segnum == 0)
-    bez->points[0].p3 = bez->points[0].p1;
-  prev.x =
-    coeff[0][0] * bez->points[segnum].p3.x +
-    coeff[0][1] * bez->points[segnum+1].p1.x +
-    coeff[0][2] * bez->points[segnum+1].p2.x +
-    coeff[0][3] * bez->points[segnum+1].p3.x;
-  prev.y =
-    coeff[0][0] * bez->points[segnum].p3.y +
-    coeff[0][1] * bez->points[segnum+1].p1.y +
-    coeff[0][2] * bez->points[segnum+1].p2.y +
-    coeff[0][3] * bez->points[segnum+1].p3.y;
-  for (i = 1; i <= NSEGS; i++) {
-    pt.x =
-      coeff[i][0] * bez->points[segnum].p3.x +
-      coeff[i][1] * bez->points[segnum+1].p1.x +
-      coeff[i][2] * bez->points[segnum+1].p2.x +
-      coeff[i][3] * bez->points[segnum+1].p3.x;
-    pt.y =
-      coeff[i][0] * bez->points[segnum].p3.y +
-      coeff[i][1] * bez->points[segnum+1].p1.y +
-      coeff[i][2] * bez->points[segnum+1].p2.y +
-      coeff[i][3] * bez->points[segnum+1].p3.y;
-
-    dist = MIN(dist, distance_line_point(&prev, &pt, line_width, point));
-
-    prev = pt;
-  }
-  return dist;
-}
-
 int
 bezierconn_closest_segment(BezierConn *bez, Point *point, real line_width)
 {
+  Point last;
   int i;
   real dist = G_MAXDOUBLE;
   int closest;
 
   closest = 0;
+  last = bez->points[0].p1;
   for (i = 0; i < bez->numpoints - 1; i++) {
-    real new_dist = segment_distance(bez, point, line_width, i);
+    real new_dist = distance_bez_seg_point(&last, &bez->points[i+1].p1,
+				&bez->points[i+1].p2, &bez->points[i+1].p3,
+				line_width, point);
     if (new_dist < dist) {
       dist = new_dist;
       closest = i;
     }
+    last = bez->points[i+1].p3;
   }
   return closest;
 }
@@ -260,20 +271,24 @@ bezierconn_closest_handle(BezierConn *bez, Point *point)
   return closest;
 }
 
+Handle *
+bezierconn_closest_major_handle(BezierConn *bez, Point *point)
+{
+  Handle *closest = bezierconn_closest_handle(bez, point);
+
+  return bez->object.handles[3*get_major_nr(get_handle_nr(bez, closest))];
+}
+
 real
 bezierconn_distance_from(BezierConn *bez, Point *point, real line_width)
 {
-  int i;
-  real dist = G_MAXDOUBLE;
-  
-  for (i = 0; i < bez->numpoints - 1; i++) {
-    dist = MIN(dist, segment_distance(bez, point, line_width, i));
-  }
-  return dist;
+  return distance_bez_line_point(bez->points, bez->numpoints,
+				 line_width, point);
 }
 
 static void
-add_handles(BezierConn *bez, int pos, BezPoint *point, Handle *handle1,
+add_handles(BezierConn *bez, int pos, BezPoint *point,
+	    BezCornerType corner_type, Handle *handle1,
 	    Handle *handle2, Handle *handle3)
 {
   int i;
@@ -284,13 +299,17 @@ add_handles(BezierConn *bez, int pos, BezPoint *point, Handle *handle1,
   obj = (Object *)bez;
   bez->numpoints++;
   bez->points = g_realloc(bez->points, bez->numpoints*sizeof(BezPoint));
+  bez->corner_types = g_realloc(bez->corner_types,
+				bez->numpoints * sizeof(BezCornerType));
 
   for (i = bez->numpoints-1; i > pos; i--) {
     bez->points[i] = bez->points[i-1];
+    bez->corner_types[i] = bez->corner_types[i-1];
   }
   bez->points[pos] = *point; 
   bez->points[pos].p1 = bez->points[pos+1].p1;
-  bez->points[pos+1].p1 = point->p1;;
+  bez->points[pos+1].p1 = point->p1;
+  bez->corner_types[pos] = corner_type;
   object_add_handle_at(obj, handle1, 3*pos-2);
   object_add_handle_at(obj, handle2, 3*pos-1);
   object_add_handle_at(obj, handle3, 3*pos);
@@ -323,9 +342,12 @@ remove_handles(BezierConn *bez, int pos)
   tmppoint = bez->points[pos].p1;
   for (i = pos; i < bez->numpoints; i++) {
     bez->points[i] = bez->points[i+1];
+    bez->corner_types[i] = bez->corner_types[i+1];
   }
   bez->points[pos].p1 = tmppoint;
   bez->points = g_realloc(bez->points, bez->numpoints*sizeof(BezPoint));
+  bez->corner_types = g_realloc(bez->corner_types,
+				bez->numpoints * sizeof(BezCornerType));
 
   old_handle1 = obj->handles[3*pos-2];
   old_handle2 = obj->handles[3*pos-1];
@@ -342,6 +364,7 @@ ObjectChange *
 bezierconn_add_segment(BezierConn *bez, int segment, Point *point)
 {
   BezPoint realpoint;
+  BezCornerType corner_type = BEZ_CORNER_SYMMETRIC;
   Handle *new_handle1, *new_handle2, *new_handle3;
   Point startpoint;
 
@@ -373,12 +396,13 @@ bezierconn_add_segment(BezierConn *bez, int segment, Point *point)
   setup_corner_handle(new_handle1, HANDLE_RIGHTCTRL);
   setup_corner_handle(new_handle2, HANDLE_LEFTCTRL);
   setup_corner_handle(new_handle3, HANDLE_BEZMAJOR);
-  add_handles(bez, segment+1, &realpoint, new_handle1,new_handle2,new_handle3);
-  return bezierconn_create_change(bez, TYPE_ADD_POINT,
-				  &realpoint, segment+1,
-				  new_handle1, NULL,
-				  new_handle2, NULL,
-				  new_handle3, NULL);
+  add_handles(bez, segment+1, &realpoint, corner_type,
+	      new_handle1, new_handle2, new_handle3);
+  return bezierconn_create_point_change(bez, TYPE_ADD_POINT,
+					&realpoint, corner_type, segment+1,
+					new_handle1, NULL,
+					new_handle2, NULL,
+					new_handle3, NULL);
 }
 
 ObjectChange *
@@ -387,13 +411,18 @@ bezierconn_remove_segment(BezierConn *bez, int pos)
   Handle *old_handle1, *old_handle2, *old_handle3;
   ConnectionPoint *cpt1, *cpt2, *cpt3;
   BezPoint old_point;
+  BezCornerType old_ctype;
 
   assert(pos > 0);
+  assert(bez->numpoints > 2);
+
+  if (pos == bez->numpoints-1) pos--;
 
   old_handle1 = bez->object.handles[3*pos-2];
   old_handle2 = bez->object.handles[3*pos-1];
   old_handle3 = bez->object.handles[3*pos];
   old_point = bez->points[pos];
+  old_ctype = bez->corner_types[pos];
 
   cpt1 = old_handle1->connected_to;
   cpt2 = old_handle2->connected_to;
@@ -407,11 +436,108 @@ bezierconn_remove_segment(BezierConn *bez, int pos)
 
   bezierconn_update_data(bez);
   
-  return bezierconn_create_change(bez, TYPE_REMOVE_POINT,
-				  &old_point, pos,
-				  old_handle1, cpt1,
-				  old_handle2, cpt2,
-				  old_handle3, cpt3);
+  return bezierconn_create_point_change(bez, TYPE_REMOVE_POINT,
+					&old_point, old_ctype, pos,
+					old_handle1, cpt1,
+					old_handle2, cpt2,
+					old_handle3, cpt3);
+}
+
+static void
+bezierconn_straighten_corner(BezierConn *bez, int comp_nr) {
+  /* Neat thing would be to have the kind of straigthening depend on
+     which handle was chosen:  Mid-handle does average, other leaves that
+     handle where it is. */
+  switch (bez->corner_types[comp_nr]) {
+  case BEZ_CORNER_SYMMETRIC: {
+    Point pt1, pt2;
+    pt1 = bez->points[comp_nr].p3;
+    point_sub(&pt1, &bez->points[comp_nr].p2);
+    pt2 = bez->points[comp_nr].p3;
+    point_sub(&pt2, &bez->points[comp_nr+1].p1);
+    point_scale(&pt2, -1.0);
+    point_add(&pt1, &pt2);
+    point_scale(&pt1, 0.5);
+    pt2 = pt1;
+    point_scale(&pt1, -1.0);
+    point_add(&pt1, &bez->points[comp_nr].p3);
+    point_add(&pt2, &bez->points[comp_nr].p3);
+    bez->points[comp_nr].p2 = pt1;
+    bez->points[comp_nr+1].p1 = pt2;
+    bezierconn_update_data(bez);
+  }
+  break;
+  case BEZ_CORNER_SMOOTH: {
+    Point pt1, pt2;
+    real len1, len2;
+    pt1 = bez->points[comp_nr].p3;
+    point_sub(&pt1, &bez->points[comp_nr].p2);
+    pt2 = bez->points[comp_nr].p3;
+    point_sub(&pt2, &bez->points[comp_nr+1].p1);
+    len1 = point_len(&pt1);
+    len2 = point_len(&pt2);
+    point_scale(&pt2, -1.0);
+    if (len1 > 0)
+      point_normalize(&pt1);
+    if (len2 > 0)
+      point_normalize(&pt2);
+    point_add(&pt1, &pt2);
+    point_scale(&pt1, 0.5);
+    pt2 = pt1;
+    point_scale(&pt1, -len1);
+    point_add(&pt1, &bez->points[comp_nr].p3);
+    point_scale(&pt2, len2);
+    point_add(&pt2, &bez->points[comp_nr].p3);
+    bez->points[comp_nr].p2 = pt1;
+    bez->points[comp_nr+1].p1 = pt2;
+    bezierconn_update_data(bez);
+  }
+    break;
+  case BEZ_CORNER_CUSP:
+    break;
+  }
+}
+
+ObjectChange *
+bezierconn_set_corner_type(BezierConn *bez, Handle *handle,
+			   BezCornerType corner_type)
+{
+  Handle *mid_handle;
+  Point old_left, old_right;
+  int old_type;
+  int handle_nr, comp_nr;
+
+  handle_nr = get_handle_nr(bez, handle);
+
+  switch (handle->id) {
+  case HANDLE_BEZMAJOR:
+    mid_handle = handle;
+    break;
+  case HANDLE_LEFTCTRL:
+    handle_nr++;
+    mid_handle = bez->object.handles[handle_nr];
+    break;
+  case HANDLE_RIGHTCTRL:
+    handle_nr--;
+    mid_handle = bez->object.handles[handle_nr];
+    break;
+  default:
+    message_warning(_("Internal error: Setting corner type of endpoint of bezier"));
+    return NULL;
+  }
+
+  comp_nr = get_major_nr(handle_nr);
+
+  old_type = bez->corner_types[comp_nr];
+  old_left = bez->points[comp_nr].p2;
+  old_right = bez->points[comp_nr+1].p1;
+
+  bez->corner_types[comp_nr] = corner_type;
+
+  bezierconn_straighten_corner(bez, comp_nr);
+
+  return bezierconn_create_corner_change(bez, mid_handle, &old_left, &old_right,
+					 old_type, corner_type);
 }
 
 void
@@ -496,7 +622,7 @@ bezierconn_draw_control_lines(BezierConn *bez, Renderer *renderer)
   Color line_colour = {0.0, 0.0, 0.6};
   Point startpoint;
   int i;
-
+  
   /* setup renderer ... */
   renderer->ops->set_linewidth(renderer, 0);
   renderer->ops->set_linestyle(renderer, LINESTYLE_DOTTED);
@@ -529,6 +655,10 @@ bezierconn_init(BezierConn *bez)
   bez->points[0].type = BEZ_MOVE_TO;
   bez->points[1].type = BEZ_CURVE_TO;
 
+  bez->corner_types = g_new(BezCornerType, 2);
+  bez->corner_types[0] = BEZ_CORNER_SYMMETRIC;
+  bez->corner_types[1] = BEZ_CORNER_SYMMETRIC;
+
   bez->object.handles[0] = g_new(Handle, 1);
   bez->object.handles[1] = g_new(Handle, 1);
   bez->object.handles[2] = g_new(Handle, 1);
@@ -571,9 +701,11 @@ bezierconn_copy(BezierConn *from, BezierConn *to)
   to->numpoints = from->numpoints;
 
   to->points = g_new(BezPoint, to->numpoints);
+  to->corner_types = g_new(BezCornerType, to->numpoints);
 
   for (i = 0; i < to->numpoints; i++) {
     to->points[i] = from->points[i];
+    to->corner_types[i] = from->corner_types[i];
   }
 
   to->object.handles[0] = g_new(Handle,1);
@@ -609,6 +741,7 @@ bezierconn_destroy(BezierConn *bez)
   g_free(temp_handles);
   
   g_free(bez->points);
+  g_free(bez->corner_types);
 }
 
 
@@ -628,6 +761,10 @@ bezierconn_save(BezierConn *bez, ObjectNode obj_node)
     data_add_point(attr, &bez->points[i].p2);
     data_add_point(attr, &bez->points[i].p3);
   }
+
+  attr = new_attribute(obj_node, "corner_types");
+  for (i = 0; i < bez->numpoints; i++)
+    data_add_enum(attr, bez->corner_types[i]);
 }
 
 void
@@ -668,6 +805,22 @@ bezierconn_load(BezierConn *bez, ObjectNode obj_node) /* NOTE: Does object_init(
     }
   }
 
+  bez->corner_types = g_new(BezCornerType, bez->numpoints);
+
+  attr = object_find_attribute(obj_node, "corner_types");
+  /* if corner_types is missing or corrupt */
+  if (!attr || attribute_num_data(attr) != bez->numpoints) {
+    for (i = 0; i < bez->numpoints; i++) {
+      bez->corner_types[i] = BEZ_CORNER_SYMMETRIC;
+    }
+  } else {
+    data = attribute_first_data(attr);
+    for (i = 0; i < bez->numpoints; i++) {
+      bez->corner_types[i] = data_enum(data);
+      data = data_next(data);
+    }
+  }
+
   obj->handles[0] = g_new(Handle, 1);
   obj->handles[0]->connect_type = HANDLE_CONNECTABLE;
   obj->handles[0]->connected_to = NULL;
@@ -692,7 +845,7 @@ bezierconn_load(BezierConn *bez, ObjectNode obj_node) /* NOTE: Does object_init(
 }
 
 static void
-bezierconn_change_free(struct PointChange *change)
+bezierconn_point_change_free(struct PointChange *change)
 {
   if ( (change->type==TYPE_ADD_POINT && !change->applied) ||
        (change->type==TYPE_REMOVE_POINT && change->applied) ){
@@ -706,12 +859,13 @@ bezierconn_change_free(struct PointChange *change)
 }
 
 static void
-bezierconn_change_apply(struct PointChange *change, Object *obj)
+bezierconn_point_change_apply(struct PointChange *change, Object *obj)
 {
   change->applied = 1;
   switch (change->type) {
   case TYPE_ADD_POINT:
     add_handles((BezierConn *)obj, change->pos, &change->point,
+		change->corner_type,
 		change->handle1, change->handle2, change->handle3);
     break;
   case TYPE_REMOVE_POINT:
@@ -724,7 +878,7 @@ bezierconn_change_apply(struct PointChange *change, Object *obj)
 }
 
 static void
-bezierconn_change_revert(struct PointChange *change, Object *obj)
+bezierconn_point_change_revert(struct PointChange *change, Object *obj)
 {
   switch (change->type) {
   case TYPE_ADD_POINT:
@@ -732,6 +886,7 @@ bezierconn_change_revert(struct PointChange *change, Object *obj)
     break;
   case TYPE_REMOVE_POINT:
     add_handles((BezierConn *)obj, change->pos, &change->point,
+		change->corner_type,
 		change->handle1, change->handle2, change->handle3);
 
     if (change->connected_to1)
@@ -747,23 +902,25 @@ bezierconn_change_revert(struct PointChange *change, Object *obj)
 }
 
 static ObjectChange *
-bezierconn_create_change(BezierConn *bez, enum change_type type,
-			 BezPoint *point, int pos,
-			 Handle *handle1, ConnectionPoint *connected_to1,
-			 Handle *handle2, ConnectionPoint *connected_to2,
-			 Handle *handle3, ConnectionPoint *connected_to3)
+bezierconn_create_point_change(BezierConn *bez, enum change_type type,
+			       BezPoint *point, BezCornerType corner_type,
+			       int pos,
+			       Handle *handle1, ConnectionPoint *connected_to1,
+			       Handle *handle2, ConnectionPoint *connected_to2,
+			       Handle *handle3, ConnectionPoint *connected_to3)
 {
   struct PointChange *change;
 
   change = g_new(struct PointChange, 1);
 
-  change->obj_change.apply = (ObjectChangeApplyFunc) bezierconn_change_apply;
-  change->obj_change.revert = (ObjectChangeRevertFunc) bezierconn_change_revert;
-  change->obj_change.free = (ObjectChangeFreeFunc) bezierconn_change_free;
+  change->obj_change.apply = (ObjectChangeApplyFunc) bezierconn_point_change_apply;
+  change->obj_change.revert = (ObjectChangeRevertFunc) bezierconn_point_change_revert;
+  change->obj_change.free = (ObjectChangeFreeFunc) bezierconn_point_change_free;
 
   change->type = type;
   change->applied = 1;
   change->point = *point;
+  change->corner_type = corner_type;
   change->pos = pos;
   change->handle1 = handle1;
   change->connected_to1 = connected_to1;
@@ -774,3 +931,56 @@ bezierconn_create_change(BezierConn *bez, enum change_type type,
 
   return (ObjectChange *)change;
 }
+
+static void
+bezierconn_corner_change_apply(struct CornerChange *change,
+			       Object *obj) {
+  BezierConn *bez = (BezierConn *)obj;
+  int handle_nr = get_handle_nr(bez, change->handle);
+  int comp_nr = get_major_nr(handle_nr);
+
+  bezierconn_straighten_corner(bez, comp_nr);
+
+  bez->corner_types[comp_nr] = change->new_type;
+
+  change->applied = 1;
+}
+
+static void
+bezierconn_corner_change_revert(struct CornerChange *change,
+				Object *obj) {
+  BezierConn *bez = (BezierConn *)obj;
+  int handle_nr = get_handle_nr(bez, change->handle);
+  int comp_nr = get_major_nr(handle_nr);
+
+  bez->points[comp_nr].p2 = change->point_left;
+  bez->points[comp_nr+1].p1 = change->point_right;
+  bez->corner_types[comp_nr] = change->old_type;  
+
+  change->applied = 0;
+}
+
+static ObjectChange *
+bezierconn_create_corner_change(BezierConn *bez, Handle *handle,
+				Point *point_left, Point *point_right,
+				BezCornerType old_corner_type,
+				BezCornerType new_corner_type)
+{
+  struct CornerChange *change;
+
+  change = g_new(struct CornerChange, 1);
+
+  change->obj_change.apply = (ObjectChangeApplyFunc) bezierconn_corner_change_apply;
+  change->obj_change.revert = (ObjectChangeRevertFunc) bezierconn_corner_change_revert;
+  change->obj_change.free = (ObjectChangeFreeFunc) NULL;
+
+  change->old_type = old_corner_type;
+  change->new_type = new_corner_type;
+  change->applied = 1;
+
+  change->handle = handle;
+  change->point_left = *point_left;
+  change->point_right = *point_right;
+
+  return (ObjectChange *)change;
+}
