@@ -32,7 +32,17 @@ extern void color_init(void);
 extern void color_convert(Color *color, GdkColor *gdkcolor);
 extern gboolean color_equals(Color *color1, Color *color2);
 
-extern Color color_black, color_white;
-extern GdkColor color_gdk_black, color_gdk_white;
+#ifdef G_OS_WIN32
+#  ifdef LIBDIA_COMPILATION
+#    define DIAVAR __declspec(dllexport)
+#  else  /* !LIBDIA_COMPILATION */
+#    define DIAVAR extern __declspec(dllimport)
+#  endif /* !LIBDIA_COMPILATION */
+#else  /* !G_OS_WIN32 */
+#  define DIAVAR extern
+#endif
+
+DIAVAR Color color_black, color_white;
+DIAVAR GdkColor color_gdk_black, color_gdk_white;
 
 #endif /* COLOR_H */
