@@ -27,6 +27,7 @@
 #include "arrows.h"
 #include "utils.h"
 #include "poly_conn.h"
+#include "polyshape.h"
 #include "bezier_conn.h"
 #include "widgets.h"
 #include "intl.h"
@@ -43,6 +44,7 @@ static void *dummy_dep[] G_GNUC_UNUSED = {
   element_update_boundingbox,
   orthconn_update_data,
   polyconn_init,
+  polyshape_init,
   bezierconn_init,
   new_text,
   font_getfont,
