@@ -187,17 +187,54 @@ typedef void (*MoveHandleFunc) (Object*          obj,
   make sure to update the values in the widgets so that it
   accurately describes the current state of the object.
   Remember to destroy this dialog when the object is destroyed!
+
+  Note that if you want to use the same dialog multiple times,
+  you should ref it first.  Just run the following on the widget
+  when you create it:
+    gtk_object_ref(GTK_OBJECT(widget));
+    gtk_object_sink(GTK_OBJECT(widget)); / * optional, but recommended * /
+  If you don't do this, the widget will be destroyed when the
+  properties dialog is closed.
 */
 typedef GtkWidget *(*GetPropertiesFunc) (Object* obj);
 
 /*
   Thiss function is called when the user clicks on
-  the "Apply" button.
+  the "Apply" button.  The widget parameter is the one created by
+  the get_properties function.
 
   Must returns a Change that can be used for undo/redo.
   The returned change is already applied.
 */
-typedef ObjectChange *(*ApplyPropertiesFunc) (Object* obj);
+typedef ObjectChange *(*ApplyPropertiesFunc) (Object* obj, GtkWidget *widget);
+
+/* properties interface functions */
+#ifndef _prop_typedefs_defined
+#define _prop_typedefs_defined
+typedef struct _PropDescription PropDescription;
+typedef struct _Property Property;
+#endif
+
+/*
+  This function is called to return a list of property
+  descriptions the object supports.  The list should be
+  NULL terminated.
+*/
+typedef const PropDescription *(* DescribePropsFunc) (Object *obj);
+
+/*
+  This function is called to return the current values
+  (and type information) for a number of properties of
+  the object.
+*/
+typedef void (* GetPropsFunc) (Object *obj, Property *props, gint nprops);
+
+/*
+  This function is called to set the value of a number
+  of properties of the object.
+*/
+typedef void (* SetPropsFunc) (Object *obj, Property *props, gint nprops);
+
 
 /*
   Function called when the user has double clicked on an Tool.
@@ -237,6 +274,9 @@ extern void object_remove_handle(Object *obj, Handle *handle);
 extern void object_add_connectionpoint(Object *obj, ConnectionPoint *conpoint);
 extern void object_remove_connectionpoint(Object *obj,
 					  ConnectionPoint *conpoint);
+extern void object_add_connectionpoint_at(Object *obj, 
+					  ConnectionPoint *conpoint,
+					  int pos);
 extern void object_connect(Object *obj, Handle *handle,
 			   ConnectionPoint *conpoint);
 extern void object_unconnect(Object *connected_obj, Handle *handle);
@@ -270,13 +310,18 @@ struct _ObjectOps {
   GetPropertiesFunc   get_properties;
   ApplyPropertiesFunc apply_properties;
   ObjectMenuFunc      get_object_menu;
+
+  DescribePropsFunc   describe_props;
+  GetPropsFunc        get_props;
+  SetPropsFunc        set_props;
+
   /*
     Unused places (for extension).
     These should be NULL for now. In the future they might be used.
     Then an older object will be binary compatible, because all new code
     checks if new ops are supported (!= NULL)
   */
-  void      (*(unused[9]))(Object *obj,...); 
+  void      (*(unused[6]))(Object *obj,...); 
 };
 
 /*
@@ -332,11 +377,22 @@ struct _ObjectType {
 		    Stored in xpm format */
   
   ObjectTypeOps *ops;
+
+  char *pixmap_file; /* fallback if pixmap is NULL */
+  void *default_user_data; /* use this if no user data is specified in
+			      the .sheet file */
 };
 
+/* base property stuff ... */
+#define OBJECT_COMMON_PROPERTIES \
+  { "obj_pos", PROP_TYPE_POINT, 0, \
+    "Object position", "Where the object is located"}, \
+  { "obj_bb", PROP_TYPE_RECT, 0, \
+    "Object bounding box", "The bounding box of the object"}
+
+#define OBJECT_COMMON_PROPERTIES_OFFSETS \
+  { "obj_pos", PROP_TYPE_POINT, offsetof(Object, position) }, \
+  { "obj_bb", PROP_TYPE_POINT, offsetof(Object, bounding_box) }
+
 #endif /* OBJECT_H */
 
-
-
-
-
