@@ -20,6 +20,7 @@
  */
 
 #include <gtk/gtk.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "geometry.h"
@@ -84,6 +85,8 @@ prop_type_register(const gchar *name, PropType_Copy cfunc,
   return ret;
 }
 
+static const PropDescription null_prop_desc = { NULL };
+
 PropDescription *
 prop_desc_lists_union(GList *plists)
 {
@@ -91,6 +94,10 @@ prop_desc_lists_union(GList *plists)
   PropDescription *ret;
   GList *tmp;
 
+  /* make sure the array is allocated */
+  g_array_append_val(arr, null_prop_desc);
+  g_array_remove_index(arr, 0);
+
   for (tmp = plists; tmp; tmp = tmp->next) {
     PropDescription *plist = tmp->data;
     int i;
@@ -117,6 +124,10 @@ prop_desc_lists_intersection(GList *plists)
   GList *tmp;
   gint i;
 
+  /* make sure the array is allocated */
+  g_array_append_val(arr, null_prop_desc);
+  g_array_remove_index(arr, 0);
+
   if (plists) {
     ret = plists->data;
     for (i = 0; ret[i].name != NULL; i++)
@@ -315,7 +326,7 @@ prop_get_widget(Property *prop)
 	gtk_widget_show(item);
       }
       gtk_option_menu_set_menu(GTK_OPTION_MENU(ret), menu);
-      gtk_option_menu_set_history(GTK_OPTION_MENU(ret), i);
+      gtk_option_menu_set_history(GTK_OPTION_MENU(ret), pos);
     } else {
       ret = gtk_entry_new(); /* should use spin button/option menu */
       g_snprintf(buf, sizeof(buf), "%d", PROP_VALUE_ENUM(*prop));
@@ -968,6 +979,8 @@ object_create_props_dialog(Object *obj)
   g_return_val_if_fail(obj->ops->set_props != NULL, NULL);
 
   pdesc = obj->ops->describe_props(obj);
+  g_return_val_if_fail(pdesc != NULL, NULL);
+
   for (i = 0; pdesc[i].name != NULL; i++)
     if ((pdesc[i].flags & PROP_FLAG_VISIBLE) != 0)
       nprops++;
@@ -1095,6 +1108,9 @@ object_save_props(Object *obj, ObjectNode obj_node)
 
 /* --------------------------------------- */
 
+#ifdef G_OS_WIN32
+/* moved to properties.h ... */
+#else
 /* standard property extra data members */
 PropNumData prop_std_line_width_data = { 0.0, 10.0, 0.01 };
 PropNumData prop_std_text_height_data = { 0.1, 10.0, 0.1 };
@@ -1104,3 +1120,4 @@ PropEnumData prop_std_text_align_data[] = {
   { N_("Right"), ALIGN_RIGHT },
   { NULL, 0 }
 };
+#endif
\ No newline at end of file
