@@ -252,6 +252,8 @@ relationship_get_properties(Relationship *relationship)
     relationship->properties_dialog = prop_dialog;
 
     vbox = gtk_vbox_new(FALSE, 5);
+    gtk_object_ref(GTK_OBJECT(vbox));
+    gtk_object_sink(GTK_OBJECT(vbox));
     prop_dialog->vbox = vbox;
 
     hbox = gtk_hbox_new(FALSE, 5);
