@@ -725,29 +725,36 @@ load_shape_info(const gchar *filename)
 {
   xmlDocPtr doc = xmlParseFile(filename);
   xmlNsPtr shape_ns, svg_ns;
-  xmlNodePtr node;
+  xmlNodePtr node,root;
   ShapeInfo *info;
   char *tmp;
+#if 0
   int descr_score = -1;
+#endif
   
   if (!doc) {
     g_warning("parse error for %s", filename);
     return NULL;
   }
-  if (!(shape_ns = xmlSearchNsByHref(doc, doc->root,
+  /* skip (emacs) comments */
+  root = doc->root;
+  while (root && (root->type != XML_ELEMENT_NODE)) root = root->next;
+  if (!root) return NULL;
+
+  if (!(shape_ns = xmlSearchNsByHref(doc, root,
 		"http://www.daa.com.au/~james/dia-shape-ns"))) {
     xmlFreeDoc(doc);
     g_warning("could not find shape namespace");
     return NULL;
   }
-  if (!(svg_ns = xmlSearchNsByHref(doc, doc->root,
+  if (!(svg_ns = xmlSearchNsByHref(doc, root,
 		"http://www.w3.org/Graphics/SVG/svg-19990730.dtd"))) {
     xmlFreeDoc(doc);
     g_warning("could not find svg namespace");
     return NULL;
   }
-  if (doc->root->ns != shape_ns || strcmp(doc->root->name, "shape")) {
-    g_warning("root element was %s -- expecting shape", doc->root->name);
+  if (root->ns != shape_ns || strcmp(root->name, "shape")) {
+    g_warning("root element was %s -- expecting shape", root->name);
     xmlFreeDoc(doc);
     return NULL;
   }
@@ -758,7 +765,7 @@ load_shape_info(const gchar *filename)
   info->shape_bounds.bottom = -DBL_MAX;
   info->shape_bounds.right = -DBL_MAX;
 
-  for (node = doc->root->childs; node != NULL; node = node->next) {
+  for (node = root->childs; node != NULL; node = node->next) {
     if (node->type != XML_ELEMENT_NODE)
       continue;
     if (node->ns == shape_ns && !strcmp(node->name, "name")) {
@@ -766,6 +773,7 @@ load_shape_info(const gchar *filename)
       g_free(info->name);
       info->name = g_strdup(tmp);
       free(tmp);
+#if 0
     } else if (node->ns == shape_ns && !strcmp(node->name, "description")) {
       gint score;
 
@@ -784,6 +792,7 @@ load_shape_info(const gchar *filename)
 	info->description = g_strdup(tmp);
 	free(tmp);
       }
+#endif
     } else if (node->ns == shape_ns && !strcmp(node->name, "icon")) {
       tmp = xmlNodeGetContent(node);
       g_free(info->icon);
@@ -890,7 +899,9 @@ shape_info_print(ShapeInfo *info)
   GList *tmp;
   int i;
   g_print("Name        : %s\n", info->name);
+#if 0
   g_print("Description : %s\n", info->description);
+#endif
   g_print("Connections :\n");
   for (i = 0; i < info->nconnections; i++)
     g_print("  (%g, %g)\n", info->connections[i].x, info->connections[i].y);
