@@ -32,6 +32,7 @@
 #include "text.h"
 #include "widgets.h"
 #include "message.h"
+#include "properties.h"
 
 #include "pixmaps/box.xpm"
 
@@ -46,24 +47,7 @@ typedef enum {
 } AnchorShape;
 
 typedef struct _Box Box;
-typedef struct _BoxPropertiesDialog BoxPropertiesDialog;
 typedef struct _BoxDefaultsDialog BoxDefaultsDialog;
-typedef struct _BoxState BoxState;
-
-struct _BoxState {
-  ObjectState obj_state;
-  
-  real border_width;
-  Color border_color;
-  Color inner_color;
-  gboolean show_background;
-  LineStyle line_style;
-  real dashlength;
-  real corner_radius;
-
-  real padding;
-  TextAttributes text_attrib;
-};
 
 struct _Box {
   Element element;
@@ -94,24 +78,6 @@ typedef struct _BoxProperties {
   Color *font_color;
 } BoxProperties;
 
-struct _BoxPropertiesDialog {
-  GtkWidget *vbox;
-
-  GtkSpinButton *border_width;
-  DiaColorSelector *fg_color;
-  DiaColorSelector *bg_color;
-  GtkToggleButton *show_background;
-  DiaLineStyleSelector *line_style;
-  GtkSpinButton *corner_radius;
-
-  GtkSpinButton *padding;
-  DiaFontSelector *font;
-  GtkSpinButton *font_size;
-  DiaColorSelector *font_color;
-
-  Box *box;
-};
-
 struct _BoxDefaultsDialog {
   GtkWidget *vbox;
 
@@ -124,7 +90,6 @@ struct _BoxDefaultsDialog {
 };
 
 
-static BoxPropertiesDialog *box_properties_dialog;
 static BoxDefaultsDialog *box_defaults_dialog;
 static BoxProperties default_properties;
 
@@ -143,11 +108,9 @@ static Object *box_create(Point *startpoint,
 			  Handle **handle2);
 static void box_destroy(Box *box);
 static Object *box_copy(Box *box);
-static GtkWidget *box_get_properties(Box *box);
-static ObjectChange *box_apply_properties(Box *box);
-
-static BoxState *box_get_state(Box *box);
-static void box_set_state(Box *box, BoxState *state);
+static PropDescription *box_describe_props(Box *box);
+static void box_get_props(Box *box, Property *props, guint nprops);
+static void box_set_props(Box *box, Property *props, guint nprops);
 
 static void box_save(Box *box, ObjectNode obj_node, const char *filename);
 static Object *box_load(ObjectNode obj_node, int version, const char *filename);
@@ -180,217 +143,123 @@ static ObjectOps box_ops = {
   (CopyFunc)            box_copy,
   (MoveFunc)            box_move,
   (MoveHandleFunc)      box_move_handle,
-  (GetPropertiesFunc)   box_get_properties,
-  (ApplyPropertiesFunc) box_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   box_describe_props,
+  (GetPropsFunc)        box_get_props,
+  (SetPropsFunc)        box_set_props,
 };
 
-static ObjectChange *
-box_apply_properties(Box *box)
+static PropNumData corner_radius_data = { 0.0, 10.0, 0.1 };
+static PropNumData text_padding_data = { 0.0, 10.0, 0.1 };
+
+static PropDescription box_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  PROP_STD_LINE_WIDTH,
+  PROP_STD_LINE_COLOUR,
+  PROP_STD_FILL_COLOUR,
+  PROP_STD_SHOW_BACKGROUND,
+  PROP_STD_LINE_STYLE,
+  { "corner_radius", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
+    N_("Corner radius"), NULL, &corner_radius_data },
+  { "padding", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
+    N_("Text padding"), NULL, &text_padding_data },
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT,
+  
+  { NULL, 0, 0, NULL, NULL, NULL, 0}
+};
+
+static PropDescription *
+box_describe_props(Box *box)
 {
-  ObjectState *old_state;
-  Font *font;
-  Color col;
+  if (box_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(box_props);
+  return box_props;
+}
 
-  if (box != box_properties_dialog->box) {
-    message_warning("Box dialog problem:  %p != %p\n", box,
-		    box_properties_dialog->box);
-    box = box_properties_dialog->box;
+static PropOffset box_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { "line_width", PROP_TYPE_REAL, offsetof(Box, border_width) },
+  { "line_colour", PROP_TYPE_COLOUR, offsetof(Box, border_color) },
+  { "fill_colour", PROP_TYPE_COLOUR, offsetof(Box, inner_color) },
+  { "show_background", PROP_TYPE_BOOL, offsetof(Box, show_background) },
+  { "line_style", PROP_TYPE_LINESTYLE,
+    offsetof(Box, line_style), offsetof(Box, dashlength) },
+  { "corner_radius", PROP_TYPE_REAL, offsetof(Box, corner_radius) },
+  { "padding", PROP_TYPE_REAL, offsetof(Box, padding) },
+  { NULL, 0, 0 },
+};
+
+static struct { const gchar *name; GQuark q; } quarks[] = {
+  { "text_font" },
+  { "text_height" },
+  { "text_colour" },
+  { "text" }
+};
+
+static void
+box_get_props(Box *box, Property *props, guint nprops)
+{
+  guint i;
+
+  if (object_get_props_from_offsets((Object *)box, box_offsets, props, nprops))
+    return;
+  /* these props can't be handled as easily */
+  if (quarks[0].q == 0)
+    for (i = 0; i < 4; i++)
+      quarks[i].q = g_quark_from_static_string(quarks[i].name);
+  for (i = 0; i < nprops; i++) {
+    GQuark pquark = g_quark_from_string(props[i].name);
+
+    if (pquark == quarks[0].q) {
+      props[i].type = PROP_TYPE_FONT;
+      PROP_VALUE_FONT(props[i]) = box->text->font;
+    } else if (pquark == quarks[1].q) {
+      props[i].type = PROP_TYPE_REAL;
+      PROP_VALUE_REAL(props[i]) = box->text->height;
+    } else if (pquark == quarks[2].q) {
+      props[i].type = PROP_TYPE_COLOUR;
+      PROP_VALUE_COLOUR(props[i]) = box->text->color;
+    } else if (pquark == quarks[3].q) {
+      props[i].type = PROP_TYPE_STRING;
+      g_free(PROP_VALUE_STRING(props[i]));
+      PROP_VALUE_STRING(props[i]) = text_get_string_copy(box->text);
+    }
   }
+}
 
-  old_state = (ObjectState *)box_get_state(box);
-  
-  box->border_width = gtk_spin_button_get_value_as_float(box_properties_dialog->border_width);
-  dia_color_selector_get_color(box_properties_dialog->fg_color, &box->border_color);
-  dia_color_selector_get_color(box_properties_dialog->bg_color, &box->inner_color);
-  box->show_background = gtk_toggle_button_get_active(box_properties_dialog->show_background);
-  dia_line_style_selector_get_linestyle(box_properties_dialog->line_style, &box->line_style, &box->dashlength);
-  box->corner_radius = gtk_spin_button_get_value_as_float(box_properties_dialog->corner_radius);
+static void
+box_set_props(Box *box, Property *props, guint nprops)
+{
+  if (!object_set_props_from_offsets((Object *)box, box_offsets,
+				     props, nprops)) {
+    guint i;
 
-  box->padding = gtk_spin_button_get_value_as_float(box_properties_dialog->padding);
-  font = dia_font_selector_get_font(box_properties_dialog->font);
-  text_set_font(box->text, font);
-  text_set_height(box->text, gtk_spin_button_get_value_as_float(
-					box_properties_dialog->font_size));
-  dia_color_selector_get_color(box_properties_dialog->font_color, &col);
-  text_set_color(box->text, &col);
-  
+    if (quarks[0].q == 0)
+      for (i = 0; i < 4; i++)
+	quarks[i].q = g_quark_from_static_string(quarks[i].name);
+
+    for (i = 0; i < nprops; i++) {
+      GQuark pquark = g_quark_from_string(props[i].name);
+
+      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_FONT) {
+	text_set_font(box->text, PROP_VALUE_FONT(props[i]));
+      } else if (pquark == quarks[1].q && props[i].type == PROP_TYPE_REAL) {
+	text_set_height(box->text, PROP_VALUE_REAL(props[i]));
+      } else if (pquark == quarks[2].q && props[i].type == PROP_TYPE_COLOUR) {
+	text_set_color(box->text, &PROP_VALUE_COLOUR(props[i]));
+      } else if (pquark == quarks[3].q && props[i].type == PROP_TYPE_STRING) {
+	text_set_string(box->text, PROP_VALUE_STRING(props[i]));
+      }
+    }
+  }
   box_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
-  return new_object_state_change((Object *)box, old_state, 
-				 (GetStateFunc)box_get_state,
-				 (SetStateFunc)box_set_state);
 }
 
-static GtkWidget *
-box_get_properties(Box *box)
-{
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GtkWidget *color;
-  GtkWidget *checkbox;
-  GtkWidget *linestyle;
-  GtkWidget *border_width;
-  GtkWidget *padding;
-  GtkWidget *font;
-  GtkWidget *font_size;
-  GtkAdjustment *adj;
-
-  if (box_properties_dialog == NULL) {
-    box_properties_dialog = g_new(BoxPropertiesDialog, 1);
-
-    vbox = gtk_vbox_new(FALSE, 5);
-    box_properties_dialog->vbox = vbox;
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Border width:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.00, 10.0, 0.01, 0.0, 0.0);
-    border_width = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(border_width), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(border_width), TRUE);
-    box_properties_dialog->border_width = GTK_SPIN_BUTTON(border_width);
-    gtk_box_pack_start(GTK_BOX (hbox), border_width, TRUE, TRUE, 0);
-    gtk_widget_show (border_width);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Foreground color:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    color = dia_color_selector_new();
-    box_properties_dialog->fg_color = DIACOLORSELECTOR(color);
-    gtk_box_pack_start (GTK_BOX (hbox), color, TRUE, TRUE, 0);
-    gtk_widget_show (color);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Background color:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    color = dia_color_selector_new();
-    box_properties_dialog->bg_color = DIACOLORSELECTOR(color);
-    gtk_box_pack_start (GTK_BOX (hbox), color, TRUE, TRUE, 0);
-    gtk_widget_show (color);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Draw background"));
-    box_properties_dialog->show_background = GTK_TOGGLE_BUTTON( checkbox );
-    gtk_widget_show(checkbox);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Line style:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    linestyle = dia_line_style_selector_new();
-    box_properties_dialog->line_style = DIALINESTYLESELECTOR(linestyle);
-    gtk_box_pack_start (GTK_BOX (hbox), linestyle, TRUE, TRUE, 0);
-    gtk_widget_show (linestyle);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Corner rounding:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 0.0, 0.0);
-    border_width = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(border_width), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(border_width), TRUE);
-    box_properties_dialog->corner_radius = GTK_SPIN_BUTTON(border_width);
-    gtk_box_pack_start(GTK_BOX (hbox), border_width, TRUE, TRUE, 0);
-    gtk_widget_show (border_width);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-    gtk_widget_show (vbox);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Text padding:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 0.0, 0.0);
-    padding = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(padding), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(padding), TRUE);
-    box_properties_dialog->padding = GTK_SPIN_BUTTON(padding);
-    gtk_box_pack_start(GTK_BOX (hbox), padding, TRUE, TRUE, 0);
-    gtk_widget_show (padding);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Font:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    font = dia_font_selector_new();
-    box_properties_dialog->font = DIAFONTSELECTOR(font);
-    gtk_box_pack_start (GTK_BOX (hbox), font, TRUE, TRUE, 0);
-    gtk_widget_show (font);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Font size:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 0.0, 0.0);
-    font_size = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(font_size), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(font_size), TRUE);
-    box_properties_dialog->font_size = GTK_SPIN_BUTTON(font_size);
-    gtk_box_pack_start(GTK_BOX (hbox), font_size, TRUE, TRUE, 0);
-    gtk_widget_show (font_size);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Font color:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    color = dia_color_selector_new();
-    box_properties_dialog->font_color = DIACOLORSELECTOR(color);
-    gtk_box_pack_start (GTK_BOX (hbox), color, TRUE, TRUE, 0);
-    gtk_widget_show (color);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    gtk_widget_show (vbox);
-  }
-
-  box_properties_dialog->box = box;
-    
-  gtk_spin_button_set_value(box_properties_dialog->border_width,
-			    box->border_width);
-  dia_color_selector_set_color(box_properties_dialog->fg_color,
-			       &box->border_color);
-  dia_color_selector_set_color(box_properties_dialog->bg_color,
-			       &box->inner_color);
-  gtk_toggle_button_set_active(box_properties_dialog->show_background, 
-			       box->show_background);
-  dia_line_style_selector_set_linestyle(box_properties_dialog->line_style,
-					box->line_style, box->dashlength);
-  gtk_spin_button_set_value(box_properties_dialog->corner_radius,
-			    box->corner_radius);
-
-  gtk_spin_button_set_value(box_properties_dialog->padding,
-			    box->padding);
-  dia_font_selector_set_font(box_properties_dialog->font, box->text->font);
-  gtk_spin_button_set_value(box_properties_dialog->font_size,
-			    box->text->height);
-  dia_color_selector_set_color(box_properties_dialog->font_color,
-			       &box->text->color);
-  
-  return box_properties_dialog->vbox;
-}
 static void
 box_apply_defaults()
 {
@@ -717,45 +586,6 @@ box_draw(Box *box, Renderer *renderer)
   text_draw(box->text, renderer);
 }
 
-static BoxState *
-box_get_state(Box *box)
-{
-  BoxState *state = g_new(BoxState, 1);
-
-  state->obj_state.free = NULL;
-  
-  state->border_width = box->border_width;
-  state->border_color = box->border_color;
-  state->inner_color = box->inner_color;
-  state->show_background = box->show_background;
-  state->line_style = box->line_style;
-  state->dashlength = box->dashlength;
-  state->corner_radius = box->corner_radius;
-  state->padding = box->padding;
-  text_get_attributes(box->text, &state->text_attrib);
-
-  return state;
-}
-
-static void
-box_set_state(Box *box, BoxState *state)
-{
-  box->border_width = state->border_width;
-  box->border_color = state->border_color;
-  box->inner_color = state->inner_color;
-  box->show_background = state->show_background;
-  box->line_style = state->line_style;
-  box->dashlength = state->dashlength;
-  box->corner_radius = state->corner_radius;
-  box->padding = state->padding;
-  text_set_attributes(box->text, &state->text_attrib);
-
-  g_free(state);
-  
-  box_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
-}
-
-
 static void
 box_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
 {
@@ -982,7 +812,7 @@ box_save(Box *box, ObjectNode obj_node, const char *filename)
   if (!color_equals(&box->border_color, &color_black))
     data_add_color(new_attribute(obj_node, "border_color"),
 		   &box->border_color);
-  
+   
   if (!color_equals(&box->inner_color, &color_white))
     data_add_color(new_attribute(obj_node, "inner_color"),
 		   &box->inner_color);
@@ -997,7 +827,6 @@ box_save(Box *box, ObjectNode obj_node, const char *filename)
       box->dashlength != DEFAULT_LINESTYLE_DASHLEN)
     data_add_real(new_attribute(obj_node, "dashlength"),
                   box->dashlength);
-
   if (box->corner_radius > 0.0)
     data_add_real(new_attribute(obj_node, "corner_radius"),
 		  box->corner_radius);
@@ -1016,7 +845,7 @@ box_load(ObjectNode obj_node, int version, const char *filename)
   int i;
   AttributeNode attr;
 
-  box = g_malloc(sizeof(Box));
+  box = g_new(Box, 1);
   elem = &box->element;
   obj = (Object *) box;
   
@@ -1029,7 +858,6 @@ box_load(ObjectNode obj_node, int version, const char *filename)
   attr = object_find_attribute(obj_node, "border_width");
   if (attr != NULL)
     box->border_width =  data_real( attribute_first_data(attr) );
-
   box->border_color = color_black;
   attr = object_find_attribute(obj_node, "border_color");
   if (attr != NULL)
@@ -1065,6 +893,7 @@ box_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     box->padding =  data_real( attribute_first_data(attr) );
   
+
   box->text = NULL;
   attr = object_find_attribute(obj_node, "text");
   if (attr != NULL)
