@@ -235,6 +235,8 @@ diamond_get_properties(Diamond *diamond)
     diamond_properties_dialog = g_new(DiamondPropertiesDialog, 1);
 
     vbox = gtk_vbox_new(FALSE, 5);
+    gtk_object_ref(GTK_OBJECT(vbox));
+    gtk_object_sink(GTK_OBJECT(vbox));
     diamond_properties_dialog->vbox = vbox;
 
     hbox = gtk_hbox_new(FALSE, 5);
