@@ -243,6 +243,8 @@ pgram_get_properties(Pgram *pgram)
     pgram_properties_dialog = g_new(PgramPropertiesDialog, 1);
 
     vbox = gtk_vbox_new(FALSE, 5);
+    gtk_object_ref(GTK_OBJECT(vbox));
+    gtk_object_sink(GTK_OBJECT(vbox));
     pgram_properties_dialog->vbox = vbox;
 
     hbox = gtk_hbox_new(FALSE, 5);
