@@ -192,18 +192,23 @@ bezierline_move_handle(Bezierline *bezierline, Handle *handle,
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  bezierconn_move_handle(&bezierline->bez, handle, to, reason);
-
   if (reason == HANDLE_MOVE_CREATE || reason == HANDLE_MOVE_CREATE_FINAL) {
+    /* During creation, change the control points */
     BezierConn *bez = &bezierline->bez;
-    Point delta = bez->points[1].p3;
+    Point dist = bez->points[0].p1;
 
-    point_sub(&delta, &bez->points[0].p1);
-    point_scale(&delta, 1.0/3);
+    point_sub(&dist, to);
+    dist.y = 0;
+    point_scale(&dist, 0.332);
+
+    bezierconn_move_handle(bez, handle, to, reason);
+    
     bez->points[1].p1 = bez->points[0].p1;
-    point_add(&bez->points[1].p1, &delta);
-    bez->points[1].p2 = bez->points[1].p1;
-    point_add(&bez->points[1].p2, &delta);
+    point_sub(&bez->points[1].p1, &dist);
+    bez->points[1].p2 = *to;
+    point_add(&bez->points[1].p2, &dist);
+  } else {
+    bezierconn_move_handle(&bezierline->bez, handle, to, reason);
   }
 
   bezierline_update_data(bezierline);
@@ -250,8 +255,9 @@ bezierline_draw(Bezierline *bezierline, Renderer *renderer)
   /* these lines should only be displayed when object is selected.
    * Unfortunately the draw function is not aware of the selected
    * state.  This is a compromise until I fix this properly. */
-  if (renderer->is_interactive)
+  if (renderer->is_interactive) {
     bezierconn_draw_control_lines(&bezierline->bez, renderer);
+  }
 }
 
 static Object *
@@ -501,10 +507,31 @@ bezierline_delete_segment_callback (Object *obj, Point *clicked, gpointer data)
   return change;
 }
 
+static ObjectChange *
+bezierline_set_corner_type_callback (Object *obj, Point *clicked, gpointer data)
+{
+  Handle *closest;
+  Bezierline *bezierline = (Bezierline*) obj;
+  ObjectChange *change;
+  
+  closest = bezierconn_closest_major_handle(&bezierline->bez, clicked);
+  change = bezierconn_set_corner_type(&bezierline->bez, closest, 
+				      GPOINTER_TO_INT(data));
+  
+  bezierline_update_data(bezierline);
+  return change;
+}
 
 static DiaMenuItem bezierline_menu_items[] = {
   { N_("Add Segment"), bezierline_add_segment_callback, NULL, 1 },
   { N_("Delete Segment"), bezierline_delete_segment_callback, NULL, 1 },
+  { NULL, NULL, NULL, 1 },
+  { N_("Symmetric control"), bezierline_set_corner_type_callback, 
+    GINT_TO_POINTER(BEZ_CORNER_SYMMETRIC), 1 },
+  { N_("Smooth control"), bezierline_set_corner_type_callback, 
+    GINT_TO_POINTER(BEZ_CORNER_SMOOTH), 1 },
+  { N_("Cusp control"), bezierline_set_corner_type_callback,
+    GINT_TO_POINTER(BEZ_CORNER_CUSP), 1 },
 };
 
 static DiaMenu bezierline_menu = {
@@ -517,8 +544,32 @@ static DiaMenu bezierline_menu = {
 static DiaMenu *
 bezierline_get_object_menu(Bezierline *bezierline, Point *clickedpoint)
 {
+  Handle *closest;
+  gboolean closest_is_endpoint;
+  BezCornerType ctype = 42; /* nothing */
+  gint i;
+
+  closest = bezierconn_closest_major_handle(&bezierline->bez, clickedpoint);
+  if (closest->id == HANDLE_MOVE_STARTPOINT ||
+      closest->id == HANDLE_MOVE_ENDPOINT)
+    closest_is_endpoint = TRUE;
+  else
+    closest_is_endpoint = FALSE;
+
+  for (i = 0; i < bezierline->bez.numpoints; i++)
+    if (bezierline->bez.object.handles[3*i] == closest) {
+      ctype = bezierline->bez.corner_types[i];
+      break;
+    }
+
   /* Set entries sensitive/selected etc here */
   bezierline_menu_items[0].active = 1;
   bezierline_menu_items[1].active = bezierline->bez.numpoints > 2;
+  bezierline_menu_items[3].active = !closest_is_endpoint &&
+    (ctype != BEZ_CORNER_SYMMETRIC);
+  bezierline_menu_items[4].active = !closest_is_endpoint &&
+    (ctype != BEZ_CORNER_SMOOTH);
+  bezierline_menu_items[5].active = !closest_is_endpoint &&
+    (ctype != BEZ_CORNER_CUSP);
   return &bezierline_menu;
 }
